// Middleman.m

//Copyright (C) 2001 by Guido Fioretti, via Angeloni 11, I - 05100 Terni. This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

#import "Middleman.h"
#import "ModelSwarm.h"
#import "ObserverSwarm.h"
#import <random.h>






@implementation Middleman




// This method enables ModelSwarm to initialize a Middleman's position. Global variables xPosEast, xPosWest, yPosNorth, yPosSouth, xPosEastEast, yPosNorthNorth, xPosWestWest and yPosSouthSouth are also initialized here.

- setX: (int)x Y: (int)y
{
  xPos = x;
  yPos = y;
  
  xPosEast = (xPos + 1 + worldXSize) % worldXSize;
  yPosNorth = (yPos + 1 + worldYSize) % worldYSize;
  xPosWest = (xPos - 1 + worldXSize) % worldXSize;
  yPosSouth = (yPos - 1 + worldYSize) % worldYSize;
  
  xPosEastEast = (xPosEast + 1 + worldXSize) % worldXSize;
  yPosNorthNorth = (yPosNorth + 1 + worldYSize) % worldYSize;
  xPosWestWest = (xPosWest - 1 + worldXSize) % worldXSize;
  yPosSouthSouth = (yPosSouth - 1 + worldYSize) % worldYSize;

  return self;
}




// These methods make xPos, yPos, xPosEast, xPosWest, yPosNorth and yPosSouth available to other firms.

- (int)getXPos
{
  return xPos;
}

- (int)getYPos
{
  return yPos;
}

- (int)getXPosEast
{
  return xPosEast;
}

- (int)getXPosWest
{
  return xPosWest;
}

- (int)getYPosNorth
{
  return yPosNorth;
}

- (int)getYPosSouth
{
  return yPosSouth;
}




// This method enables ModelSwarm to give an identity to each instance of Middleman that it creates.

- setIdentity: (int)ID
{
  identity = ID;
  return self;
}




// This method allows Middleman to ask other Middleman about their identity.

- (int)getIdentity
{
  return identity;
}




// This method enables ModelSwarm to make the grid visible to Middleman. As soon as the grid is visible, Middleman imports its size.

- setWorld: W
{
  world = W;

  worldXSize = [world getSizeX];
  worldYSize = [world getSizeY];
  
  return self;
}




// This method enables Middleman to see ModelSwarm. In this way, it can ask ModelSwarm for the positions where production chains are, and not jump into them.  In order to avoid cross-references, ModelSwarm.h is imported in Middleman.m. This method is activated by -firmsCreateAndDestroy of ModelSwarm.

- setModelSwarm: MS
{
  modelSwarm = MS;
  return self;
}




// This method enables Middleman to see ObserverSwarm. This is necessary in order to communicate the data that enter the graph. Since the instances of Middleman are created in Modelswarm, this method is activated from ModelSwarm. This is possible because - thanks to a similar method - ModelSwarm is able to see ObserverSwarm. In order to avoid cross-references, ObserverSwarm.h is imported in Middleman.m and Middleman.h is imported in ObserverSwarm.h. Object observerSwarm is declared in Middleman.h.

- setObserverSwarm: OS
{
  observerSwarm = OS;
  return self;
}




// This method creates the list where Middleman stores the firms that are employed in production chains during a step.

- buildObjects
{
  employedFirmsList = [List create: [self getZone]];
  return self;
}    




// This method ends the creation of Middleman. It can be used to initialize global variables.

- createEnd
{
  [super createEnd];

  workInProgressEast = 0;
  workInProgressNorth = 0;
  workInProgressWest = 0;
  workInProgressSouth = 0;
  
  [employedFirmsList addFirst: nil];

  return self;
}




// When asked who they are, Middleman answer "10".

- (int)whoAreYou
{
  return 10;
}




// These methods say whether the positions surrounding Middleman are free.  Method -doYouHaveSpaceAround returns 1 if at least one out of the four positions surrounding Middleman is free and it is surrounded by free positions, 0 otherwise. Method -doYouHaveFreeSpaceEast returns 1 if the position east of Middleman as well as the three positions that surround it are free, 0 otherwise. Methods -doYouHaveFreeSpaceNorth, -doYouHaveFreeSpaceWest and -doYouHaveFreeSpaceSouth behave similarly.  Unlike other firms, Middleman needs to update its neighbors within these methods, because other firms can move aside a Middleman by their own initiative.  Furthermore, Middleman's neighbor can be declared as local variables because they are not needed by any other method.

- (int)doYouHaveFreeSpaceAround
{
  int freeSpaceAround;

  id eastOfMiddleman;
  id northOfMiddleman;
  id westOfMiddleman;
  id southOfMiddleman;

  id eastEastOfMiddleman;
  id northNorthOfMiddleman;
  id westWestOfMiddleman;
  id southSouthOfMiddleman;

  id northEastOfMiddleman;
  id northWestOfMiddleman;
  id southWestOfMiddleman;
  id southEastOfMiddleman;

  eastOfMiddleman = [world getObjectAtX: xPosEast Y: yPos];
  northOfMiddleman = [world getObjectAtX: xPos Y: yPosNorth];
  westOfMiddleman = [world getObjectAtX: xPosWest Y: yPos];
  southOfMiddleman = [world getObjectAtX: xPos Y: yPosSouth];

  eastEastOfMiddleman = [world getObjectAtX: xPosEastEast Y: yPos];
  northNorthOfMiddleman = [world getObjectAtX: xPos Y: yPosNorthNorth];
  westWestOfMiddleman = [world getObjectAtX: xPosWestWest Y: yPos];
  southSouthOfMiddleman = [world getObjectAtX: xPos Y: yPosSouthSouth];

  northEastOfMiddleman = [world getObjectAtX: xPosEast Y: yPosNorth];
  northWestOfMiddleman = [world getObjectAtX: xPosWest Y: yPosNorth];
  southWestOfMiddleman = [world getObjectAtX: xPosWest Y: yPosSouth];
  southEastOfMiddleman = [world getObjectAtX: xPosEast Y: yPosSouth];

  if (((eastOfMiddleman == nil) && (southEastOfMiddleman == nil) && (eastEastOfMiddleman == nil) && (northEastOfMiddleman == nil)) || ((northOfMiddleman == nil) && (northEastOfMiddleman == nil) && (northNorthOfMiddleman == nil) && (northWestOfMiddleman == nil)) || ((westOfMiddleman == nil) && (northWestOfMiddleman == nil) && (westWestOfMiddleman == nil) && (southWestOfMiddleman == nil)) || ((southOfMiddleman == nil) && (southWestOfMiddleman == nil) && (southSouthOfMiddleman == nil) && (southEastOfMiddleman == nil)))
    freeSpaceAround = 1;
  else
    freeSpaceAround = 0;

  return freeSpaceAround;
}

- (int)doYouHaveFreeSpaceEast
{
  int freeSpaceEast;

  id eastOfMiddleman;
  id eastEastOfMiddleman;
  id northEastOfMiddleman;
  id southEastOfMiddleman;

  eastOfMiddleman = [world getObjectAtX: xPosEast Y: yPos];
  eastEastOfMiddleman = [world getObjectAtX: xPosEastEast Y: yPos];
  northEastOfMiddleman = [world getObjectAtX: xPosEast Y: yPosNorth];
  southEastOfMiddleman = [world getObjectAtX: xPosEast Y: yPosSouth];

  if ((eastOfMiddleman == nil) && (southEastOfMiddleman == nil) && (eastEastOfMiddleman == nil) && (northEastOfMiddleman == nil))
    freeSpaceEast = 1;
  else
    freeSpaceEast = 0;
  
  return freeSpaceEast;
}

- (int)doYouHaveFreeSpaceNorth
{
  int freeSpaceNorth;

  id northOfMiddleman;
  id northNorthOfMiddleman;
  id northEastOfMiddleman;
  id northWestOfMiddleman;

  northOfMiddleman = [world getObjectAtX: xPos Y: yPosNorth];
  northNorthOfMiddleman = [world getObjectAtX: xPos Y: yPosNorthNorth];
  northEastOfMiddleman = [world getObjectAtX: xPosEast Y: yPosNorth];
  northWestOfMiddleman = [world getObjectAtX: xPosWest Y: yPosNorth];

  if ((northOfMiddleman == nil) && (northEastOfMiddleman == nil) && (northNorthOfMiddleman == nil) && (northWestOfMiddleman == nil))
    freeSpaceNorth = 1;
  else
    freeSpaceNorth = 0;
  
  return freeSpaceNorth;
}

- (int)doYouHaveFreeSpaceWest
{
  int freeSpaceWest;

  id westOfMiddleman;
  id westWestOfMiddleman;
  id northWestOfMiddleman;
  id southWestOfMiddleman;

  westOfMiddleman = [world getObjectAtX: xPosWest Y: yPos];
  westWestOfMiddleman = [world getObjectAtX: xPosWestWest Y: yPos];
  northWestOfMiddleman = [world getObjectAtX: xPosWest Y: yPosNorth];
  southWestOfMiddleman = [world getObjectAtX: xPosWest Y: yPosSouth];

  if ((westOfMiddleman == nil) && (northWestOfMiddleman == nil) && (westWestOfMiddleman == nil) && (southWestOfMiddleman == nil))
    freeSpaceWest = 1;
  else
    freeSpaceWest = 0;
  
  return freeSpaceWest;
}

- (int)doYouHaveFreeSpaceSouth
{
  int freeSpaceSouth;

  id southOfMiddleman;
  id southSouthOfMiddleman;
  id southWestOfMiddleman;
  id southEastOfMiddleman;

  southOfMiddleman = [world getObjectAtX: xPos Y: yPosSouth];
  southSouthOfMiddleman = [world getObjectAtX: xPos Y: yPosSouthSouth];
  southWestOfMiddleman = [world getObjectAtX: xPosWest Y: yPosSouth];
  southEastOfMiddleman = [world getObjectAtX: xPosEast Y: yPosSouth];
  
  if ((southOfMiddleman == nil) && (southWestOfMiddleman == nil) && (southSouthOfMiddleman == nil) && (southEastOfMiddleman == nil))
    freeSpaceSouth = 1;
  else
    freeSpaceSouth = 0;

  return freeSpaceSouth;
}




// This method allows Middleman to receive the values of workInProgress that TraderFinishedProducts may want to communicate.  Unlike other firms, Middleman has 4 workInProgress variables, one for each side a firms chain can begin from. Their names are workInProgressEast, workInProgressNorth, workInProgressWest and workInProgressSouth.  In -createEnd these variables are initialized to 0.

- setWorkInProgress: (int)wp AtSide: (int)s
{
  if (wp == 0)
    {
      if (s == 1)
	workInProgressEast = 0;
      else if (s == 2)
	workInProgressNorth = 0;
      else if (s == 3)
	workInProgressWest = 0;
      else if (s == 4)
	workInProgressSouth = 0;
    }
  else if (wp == 1)
    {
      if (s == 1)
	workInProgressEast = 1;
      else if (s == 2)
	workInProgressNorth = 1;
      else if (s == 3)
	workInProgressWest = 1;
      else if (s == 4)
	workInProgressSouth = 1;
    }
  
  return self;
}




// This method allows Middleman to tell other firms whether it is working. This information is carried by the 1-st, 2-nd, 3-rd and 4-th components of the workingState vector. The 0-th component is only used by the -areYouWorking methods of the other firms.

- (int *)areYouWorking
{
  workingState[0] = 0;
  workingState[1] = workInProgressEast;
  workingState[2] = workInProgressNorth;
  workingState[3] = workInProgressWest;
  workingState[4] = workInProgressSouth;

  return workingState;
}




// Just after a TraderFinishedProducts moved onto one of the four sides of a Middleman and activates the corresponding workInProgress variables, Middleman constructs a production chain on that side.  Technological constraints allow 11 different production chains, all of them beginning with a TraderFinishedProducts and ending with a TraderRawMaterials. If a firm that is required to complete a chain along a certain technological blueprint cannot be found, chain construction is carried out as far as possible.  Each time the -step method is called, production chains are constructed and destroyed.  Each time the -step method is called, the type of chain that is constructed depends on which firms are available next to Middleman.   Technological constraints are expressed by the following functions: (TraderFinishedProducts)<--(Finisher), (Finisher)<--(DyeingPlant, Weaver), (DyeingPlant)<--(Weaver, ComberSpinning, CarderSpinning, RagsCollector, TraderRawMaterials), (Weaver)<--(Warper), (Warper)<--(DyeingPlant, ComberSpinning, CarderSpinning, TraderRawMaterials), (ComberSpinning)<--(DyeingPlant, TraderRawMaterials), (CarderSpinning)<--(DyeingPlant, RagsCollector, TraderRawMaterials), (RagsCollector)<--(TraderRawMaterials). Production chains must not entail more than one firm of each type; furthermore, CarderSpinning and ComberSpinning exclude one another.  Each time a chain is constructed, this method communicates to ObserverSwarm which chain it is, from which side of which Middleman it begins, as well as which Finisher and which Weaver it contains. Middleman's identity is summed up with the number denoting its side. Method -communicateChain of ObserverSwarm is also called when no chain is constructed, in order to reset the variables that had been passed on.

- step
{
  int frontSideOfEastChain;
  id firstLinkOfEastChain = nil;
  id secondLinkOfEastChain = nil;
  id thirdLinkOfEastChain = nil;
  id fourthLinkOfEastChain = nil;
  id fifthLinkOfEastChain = nil;
  id sixthLinkOfEastChain = nil;
  id seventhLinkOfEastChain = nil;
  id eighthLinkOfEastChain = nil;

  int frontSideOfNorthChain;
  id firstLinkOfNorthChain = nil;
  id secondLinkOfNorthChain = nil;
  id thirdLinkOfNorthChain = nil;
  id fourthLinkOfNorthChain = nil;
  id fifthLinkOfNorthChain = nil;
  id sixthLinkOfNorthChain = nil;
  id seventhLinkOfNorthChain = nil;
  id eighthLinkOfNorthChain = nil;

  int frontSideOfWestChain;
  id firstLinkOfWestChain = nil;
  id secondLinkOfWestChain = nil;
  id thirdLinkOfWestChain = nil;
  id fourthLinkOfWestChain = nil;
  id fifthLinkOfWestChain = nil;
  id sixthLinkOfWestChain = nil;
  id seventhLinkOfWestChain = nil;
  id eighthLinkOfWestChain = nil;

  int frontSideOfSouthChain;
  id firstLinkOfSouthChain = nil;
  id secondLinkOfSouthChain = nil;
  id thirdLinkOfSouthChain = nil;
  id fourthLinkOfSouthChain = nil;
  id fifthLinkOfSouthChain = nil;
  id sixthLinkOfSouthChain = nil;
  id seventhLinkOfSouthChain = nil;
  id eighthLinkOfSouthChain = nil;

  int finisherIdentity;
  int weaverIdentity;


  if (workInProgressEast == 1)
    {
      firstLinkOfEastChain = [world getObjectAtX: xPosEast Y: yPos];
      frontSideOfEastChain = 1;
      if (firstLinkOfEastChain != nil)
	[self addFirmEast: 8 Or: 0 Or: 0 StartingFrom: firstLinkOfEastChain];
      if (firmAdded.name == 8)
	{
	  finisherIdentity = [firmAdded.pointer getIdentity];
	  secondLinkOfEastChain = firmAdded.pointer;
	  frontSideOfEastChain = firmAdded.direction;
	  if (secondLinkOfEastChain != nil)
	    {
	      if (frontSideOfEastChain == 1)
		[self addFirmEast: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfEastChain];
	      else if (frontSideOfEastChain == 2)
		[self addFirmNorth: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfEastChain];
	      else if (frontSideOfEastChain == 4)
		[self addFirmSouth: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfEastChain];
	    }	  
	  if (firmAdded.name == 6)
	    {
	      weaverIdentity = [firmAdded.pointer getIdentity];
	      thirdLinkOfEastChain = firmAdded.pointer;
	      frontSideOfEastChain = firmAdded.direction;
	      if (thirdLinkOfEastChain != nil)
		{
		  if (frontSideOfEastChain == 1)
		    [self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];
		  else if (frontSideOfEastChain == 2)
		    [self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];
		  else if (frontSideOfEastChain == 3)
		    [self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];
		  else if (frontSideOfEastChain == 4)
		    [self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];		    
		}	      
	      if (firmAdded.name == 5)
		{
		  fourthLinkOfEastChain = firmAdded.pointer;
		  frontSideOfEastChain = firmAdded.direction;
		  if (fourthLinkOfEastChain != nil)
		    {
		      if (frontSideOfEastChain == 1)
			[self addFirmEast: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfEastChain];
		      else if (frontSideOfEastChain == 2)
			[self addFirmNorth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfEastChain];
		      else if (frontSideOfEastChain == 3)
			[self addFirmWest: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfEastChain];
		      else if (frontSideOfEastChain == 4)
			[self addFirmSouth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfEastChain];
		    }
		  if (firmAdded.name == 3)
		    {
		      fifthLinkOfEastChain = firmAdded.pointer;
		      frontSideOfEastChain = firmAdded.direction;
		      if (fifthLinkOfEastChain != nil)
			{
			  if (frontSideOfEastChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  frontSideOfEastChain = firmAdded.direction;
			  if (sixthLinkOfEastChain != nil)
			    {
			      if (frontSideOfEastChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865371 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      frontSideOfEastChain = firmAdded.direction;
			      if (seventhLinkOfEastChain != nil)
				{
				  if (frontSideOfEastChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfEastChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98653721 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 4)
		    {
		      fifthLinkOfEastChain = firmAdded.pointer;
		      frontSideOfEastChain = firmAdded.direction;
		      if (fifthLinkOfEastChain != nil)
			{
			  if (frontSideOfEastChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfEastChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  frontSideOfEastChain = firmAdded.direction;
			  if (sixthLinkOfEastChain != nil)
			    {
			      if (frontSideOfEastChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865471 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 7)
		    {
		      fifthLinkOfEastChain = firmAdded.pointer;
		      frontSideOfEastChain = firmAdded.direction;
		      if (fifthLinkOfEastChain != nil)
			{
			  if (frontSideOfEastChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 986571 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  frontSideOfEastChain = firmAdded.direction;
			  if (sixthLinkOfEastChain != nil)
			    {
			      if (frontSideOfEastChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865731 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      frontSideOfEastChain = firmAdded.direction;
			      if (seventhLinkOfEastChain != nil)
				{
				  if (frontSideOfEastChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfEastChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98657321 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  frontSideOfEastChain = firmAdded.direction;
			  if (sixthLinkOfEastChain != nil)
			    {
			      if (frontSideOfEastChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865741 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else if (firmAdded.name == 7)
	    {
	      thirdLinkOfEastChain = firmAdded.pointer;
	      frontSideOfEastChain = firmAdded.direction;
	      if (thirdLinkOfEastChain != nil)
		{
		  if (frontSideOfEastChain == 1)
		    [self addFirmEast: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];
		  else if (frontSideOfEastChain == 2)
		    [self addFirmNorth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];
		  else if (frontSideOfEastChain == 3)
		    [self addFirmWest: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];
		  else if (frontSideOfEastChain == 4)
		    [self addFirmSouth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfEastChain];		    
		}	      
	      if (firmAdded.name == 6)
		{
		  weaverIdentity = [firmAdded.pointer getIdentity];
		  fourthLinkOfEastChain = firmAdded.pointer;
		  frontSideOfEastChain = firmAdded.direction;
		  if (fourthLinkOfEastChain != nil)
		    {
		      if (frontSideOfEastChain == 1)
			[self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfEastChain];
		      else if (frontSideOfEastChain == 2)
			[self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfEastChain];
		      else if (frontSideOfEastChain == 3)
			[self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfEastChain];
		      else if (frontSideOfEastChain == 4)
			[self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfEastChain];		    
		    }
		  if (firmAdded.name == 5)
		    {
		      fifthLinkOfEastChain = firmAdded.pointer;
		      frontSideOfEastChain = firmAdded.direction;
		      if (fifthLinkOfEastChain != nil)
			{
			  if (frontSideOfEastChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];
			  else if (frontSideOfEastChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfEastChain];		    
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 987651 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  frontSideOfEastChain = firmAdded.direction;
			  if (sixthLinkOfEastChain != nil)
			    {
			      if (frontSideOfEastChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfEastChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876531 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      frontSideOfEastChain = firmAdded.direction;
			      if (seventhLinkOfEastChain != nil)
				{
				  if (frontSideOfEastChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];
				  else if (frontSideOfEastChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfEastChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfEastChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98765321 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfEastChain = firmAdded.pointer;
			  frontSideOfEastChain = firmAdded.direction;
			  if (sixthLinkOfEastChain != nil)
			    {
			      if (frontSideOfEastChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];
			      else if (frontSideOfEastChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfEastChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfEastChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876541 HangingFrom: (identity + 1) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else
	    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	}
      else
	[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
      
      [self setWorkInProgress: 0 AtSide: 1];
      if (firstLinkOfEastChain != nil)
	[firstLinkOfEastChain setWorkInProgress: 0];
      if (secondLinkOfEastChain != nil)
	[secondLinkOfEastChain setWorkInProgress: 0];
      if (thirdLinkOfEastChain != nil)
	[thirdLinkOfEastChain setWorkInProgress: 0];
      if (fourthLinkOfEastChain != nil)
	[fourthLinkOfEastChain setWorkInProgress: 0];
      if (fifthLinkOfEastChain != nil)
	[fifthLinkOfEastChain setWorkInProgress: 0];
      if (sixthLinkOfEastChain != nil)
	[sixthLinkOfEastChain setWorkInProgress: 0];
      if (seventhLinkOfEastChain != nil)
	[seventhLinkOfEastChain setWorkInProgress: 0];
      if (eighthLinkOfEastChain != nil)
	[eighthLinkOfEastChain setWorkInProgress: 0];
    }
  else
    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];


  if (workInProgressNorth == 1)
    {
      firstLinkOfNorthChain = [world getObjectAtX: xPos Y: yPosNorth];
      frontSideOfNorthChain = 2;
      if (firstLinkOfNorthChain != nil)
	[self addFirmNorth: 8 Or: 0 Or: 0 StartingFrom: firstLinkOfNorthChain];
      if (firmAdded.name == 8)
	{
	  finisherIdentity = [firmAdded.pointer getIdentity];
	  secondLinkOfNorthChain = firmAdded.pointer;
	  frontSideOfNorthChain = firmAdded.direction;
	  if (secondLinkOfNorthChain != nil)
	    {
	      if (frontSideOfNorthChain == 2)
		[self addFirmNorth: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfNorthChain];
	      else if (frontSideOfNorthChain == 3)
		[self addFirmWest: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfNorthChain];
	      else if (frontSideOfNorthChain == 1)
		[self addFirmEast: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfNorthChain];
	    }
	  if (firmAdded.name == 6)
	    {
	      weaverIdentity = [firmAdded.pointer getIdentity];
	      thirdLinkOfNorthChain = firmAdded.pointer;
	      frontSideOfNorthChain = firmAdded.direction;
	      if (thirdLinkOfNorthChain != nil)
		{
		  if (frontSideOfNorthChain == 1)
		    [self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 2)
		    [self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 3)
		    [self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 4)
		    [self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];		    
		}	      
	      if (firmAdded.name == 5)
		{
		  fourthLinkOfNorthChain = firmAdded.pointer;
		  frontSideOfNorthChain = firmAdded.direction;
		  if (fourthLinkOfNorthChain != nil)
		    {
		      if (frontSideOfNorthChain == 1)
			[self addFirmEast: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfNorthChain];
		      else if (frontSideOfNorthChain == 2)
			[self addFirmNorth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfNorthChain];
		      else if (frontSideOfNorthChain == 3)
			[self addFirmWest: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfNorthChain];
		      else if (frontSideOfNorthChain == 4)
			[self addFirmSouth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfNorthChain];
		    }
		  if (firmAdded.name == 3)
		    {
		      fifthLinkOfNorthChain = firmAdded.pointer;
		      frontSideOfNorthChain = firmAdded.direction;
		      if (fifthLinkOfNorthChain != nil)
			{
			  if (frontSideOfNorthChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  frontSideOfNorthChain = firmAdded.direction;
			  if (sixthLinkOfNorthChain != nil)
			    {
			      if (frontSideOfNorthChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865371 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      frontSideOfNorthChain = firmAdded.direction;
			      if (seventhLinkOfNorthChain != nil)
				{
				  if (frontSideOfNorthChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfNorthChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98653721 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 4)
		    {
		      fifthLinkOfNorthChain = firmAdded.pointer;
		      frontSideOfNorthChain = firmAdded.direction;
		      if (fifthLinkOfNorthChain != nil)
			{
			  if (frontSideOfNorthChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfNorthChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  frontSideOfNorthChain = firmAdded.direction;
			  if (sixthLinkOfNorthChain != nil)
			    {
			      if (frontSideOfNorthChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865471 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 7)
		    {
		      fifthLinkOfNorthChain = firmAdded.pointer;
		      frontSideOfNorthChain = firmAdded.direction;
		      if (fifthLinkOfNorthChain != nil)
			{
			  if (frontSideOfNorthChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 986571 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  frontSideOfNorthChain = firmAdded.direction;
			  if (sixthLinkOfNorthChain != nil)
			    {
			      if (frontSideOfNorthChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865731 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      frontSideOfNorthChain = firmAdded.direction;
			      if (seventhLinkOfNorthChain != nil)
				{
				  if (frontSideOfNorthChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfNorthChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98657321 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  frontSideOfNorthChain = firmAdded.direction;
			  if (sixthLinkOfNorthChain != nil)
			    {
			      if (frontSideOfNorthChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865741 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else if (firmAdded.name == 7)
	    {
	      thirdLinkOfNorthChain = firmAdded.pointer;
	      frontSideOfNorthChain = firmAdded.direction;
	      if (thirdLinkOfNorthChain != nil)
		{
		  if (frontSideOfNorthChain == 1)
		    [self addFirmEast: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 2)
		    [self addFirmNorth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 3)
		    [self addFirmWest: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 4)
		    [self addFirmSouth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfNorthChain];		    
		}	      
	      if (firmAdded.name == 6)
		{
		  weaverIdentity = [firmAdded.pointer getIdentity];
		  fourthLinkOfNorthChain = firmAdded.pointer;
		  frontSideOfNorthChain = firmAdded.direction;
		  if (fourthLinkOfNorthChain != nil)
		    {
		      if (frontSideOfNorthChain == 1)
		    [self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 2)
		    [self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 3)
		    [self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfNorthChain];
		  else if (frontSideOfNorthChain == 4)
		    [self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfNorthChain];		    
		    }
		  if (firmAdded.name == 5)
		    {
		      fifthLinkOfNorthChain = firmAdded.pointer;
		      frontSideOfNorthChain = firmAdded.direction;
		      if (fifthLinkOfNorthChain != nil)
			{
			  if (frontSideOfNorthChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];
			  else if (frontSideOfNorthChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfNorthChain];		    
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 987651 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  frontSideOfNorthChain = firmAdded.direction;
			  if (sixthLinkOfNorthChain != nil)
			    {
			      if (frontSideOfNorthChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfNorthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876531 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      frontSideOfNorthChain = firmAdded.direction;
			      if (seventhLinkOfNorthChain != nil)
				{
				  if (frontSideOfNorthChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];
				  else if (frontSideOfNorthChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfNorthChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfNorthChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98765321 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfNorthChain = firmAdded.pointer;
			  frontSideOfNorthChain = firmAdded.direction;
			  if (sixthLinkOfNorthChain != nil)
			    {
			      if (frontSideOfNorthChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];
			      else if (frontSideOfNorthChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfNorthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfNorthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876541 HangingFrom: (identity + 2) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else
	    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	}
      else
	[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];

      [self setWorkInProgress: 0 AtSide: 2];
      if (firstLinkOfNorthChain != nil)
	[firstLinkOfNorthChain setWorkInProgress: 0];
      if (secondLinkOfNorthChain != nil)
	[secondLinkOfNorthChain setWorkInProgress: 0];
      if (thirdLinkOfNorthChain != nil)
	[thirdLinkOfNorthChain setWorkInProgress: 0];
      if (fourthLinkOfNorthChain != nil)
	[fourthLinkOfNorthChain setWorkInProgress: 0];
      if (fifthLinkOfNorthChain != nil)
	[fifthLinkOfNorthChain setWorkInProgress: 0];
      if (sixthLinkOfNorthChain != nil)
	[sixthLinkOfNorthChain setWorkInProgress: 0];
      if (seventhLinkOfNorthChain != nil)
	[seventhLinkOfNorthChain setWorkInProgress: 0];
      if (eighthLinkOfNorthChain != nil)
	[eighthLinkOfNorthChain setWorkInProgress: 0];
    }
  else
    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];


  if (workInProgressWest == 1)
    {
      firstLinkOfWestChain = [world getObjectAtX: xPosWest Y: yPos];
      frontSideOfWestChain = 3;
      if (firstLinkOfWestChain != nil)
	[self addFirmWest: 8 Or: 0 Or: 0 StartingFrom: firstLinkOfWestChain];
      if (firmAdded.name == 8)
	{
	  finisherIdentity = [firmAdded.pointer getIdentity];
	  secondLinkOfWestChain = firmAdded.pointer;
	  frontSideOfWestChain = firmAdded.direction;
	  if (secondLinkOfWestChain != nil)
	    {
	      if (frontSideOfWestChain == 3)
		[self addFirmWest: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfWestChain];
	      else if (frontSideOfWestChain == 4)
		[self addFirmSouth: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfWestChain];
	      else if (frontSideOfWestChain == 2)
		[self addFirmNorth: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfWestChain];
	    }
	  if (firmAdded.name == 6)
	    {
	      weaverIdentity = [firmAdded.pointer getIdentity];
	      thirdLinkOfWestChain = firmAdded.pointer;
	      frontSideOfWestChain = firmAdded.direction;
	      if (thirdLinkOfWestChain != nil)
		{
		  if (frontSideOfWestChain == 1)
		    [self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];
		  else if (frontSideOfWestChain == 2)
		    [self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];
		  else if (frontSideOfWestChain == 3)
		    [self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];
		  else if (frontSideOfWestChain == 4)
		    [self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];		    
		}	      
	      if (firmAdded.name == 5)
		{
		  fourthLinkOfWestChain = firmAdded.pointer;
		  frontSideOfWestChain = firmAdded.direction;
		  if (fourthLinkOfWestChain != nil)
		    {
		      if (frontSideOfWestChain == 1)
			[self addFirmEast: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfWestChain];
		      else if (frontSideOfWestChain == 2)
			[self addFirmNorth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfWestChain];
		      else if (frontSideOfWestChain == 3)
			[self addFirmWest: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfWestChain];
		      else if (frontSideOfWestChain == 4)
			[self addFirmSouth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfWestChain];
		    }
		  if (firmAdded.name == 3)
		    {
		      fifthLinkOfWestChain = firmAdded.pointer;
		      frontSideOfWestChain = firmAdded.direction;
		      if (fifthLinkOfWestChain != nil)
			{
			  if (frontSideOfWestChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  frontSideOfWestChain = firmAdded.direction;
			  if (sixthLinkOfWestChain != nil)
			    {
			      if (frontSideOfWestChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865371 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      frontSideOfWestChain = firmAdded.direction;
			      if (seventhLinkOfWestChain != nil)
				{
				  if (frontSideOfWestChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfWestChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98653721 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 4)
		    {
		      fifthLinkOfWestChain = firmAdded.pointer;
		      frontSideOfWestChain = firmAdded.direction;
		      if (fifthLinkOfWestChain != nil)
			{
			  if (frontSideOfWestChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfWestChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  frontSideOfWestChain = firmAdded.direction;
			  if (sixthLinkOfWestChain != nil)
			    {
			      if (frontSideOfWestChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865471 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 7)
		    {
		      fifthLinkOfWestChain = firmAdded.pointer;
		      frontSideOfWestChain = firmAdded.direction;
		      if (fifthLinkOfWestChain != nil)
			{
			  if (frontSideOfWestChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 986571 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  frontSideOfWestChain = firmAdded.direction;
			  if (sixthLinkOfWestChain != nil)
			    {
			      if (frontSideOfWestChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865731 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      frontSideOfWestChain = firmAdded.direction;
			      if (seventhLinkOfWestChain != nil)
				{
				  if (frontSideOfWestChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfWestChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98657321 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  frontSideOfWestChain = firmAdded.direction;
			  if (sixthLinkOfWestChain != nil)
			    {
			      if (frontSideOfWestChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865741 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else if (firmAdded.name == 7)
	    {
	      thirdLinkOfWestChain = firmAdded.pointer;
	      frontSideOfWestChain = firmAdded.direction;
	      if (thirdLinkOfWestChain != nil)
		{
		  if (frontSideOfWestChain == 1)
		    [self addFirmEast: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];
		  else if (frontSideOfWestChain == 2)
		    [self addFirmNorth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];
		  else if (frontSideOfWestChain == 3)
		    [self addFirmWest: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];
		  else if (frontSideOfWestChain == 4)
		    [self addFirmSouth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfWestChain];		    
		}	      
	      if (firmAdded.name == 6)
		{
		  weaverIdentity = [firmAdded.pointer getIdentity];
		  fourthLinkOfWestChain = firmAdded.pointer;
		  frontSideOfWestChain = firmAdded.direction;
		  if (fourthLinkOfWestChain != nil)
		    {
		      if (frontSideOfWestChain == 1)
			[self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfWestChain];
		      else if (frontSideOfWestChain == 2)
			[self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfWestChain];
		      else if (frontSideOfWestChain == 3)
			[self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfWestChain];
		      else if (frontSideOfWestChain == 4)
			[self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfWestChain];		    
		    }
		  if (firmAdded.name == 5)
		    {
		      fifthLinkOfWestChain = firmAdded.pointer;
		      frontSideOfWestChain = firmAdded.direction;
		      if (fifthLinkOfWestChain != nil)
			{
			  if (frontSideOfWestChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];
			  else if (frontSideOfWestChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfWestChain];		    
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 987651 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  frontSideOfWestChain = firmAdded.direction;
			  if (sixthLinkOfWestChain != nil)
			    {
			      if (frontSideOfWestChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfWestChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876531 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      frontSideOfWestChain = firmAdded.direction;
			      if (seventhLinkOfWestChain != nil)
				{
				  if (frontSideOfWestChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];
				  else if (frontSideOfWestChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfWestChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfWestChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98765321 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfWestChain = firmAdded.pointer;
			  frontSideOfWestChain = firmAdded.direction;
			  if (sixthLinkOfWestChain != nil)
			    {
			      if (frontSideOfWestChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];
			      else if (frontSideOfWestChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfWestChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfWestChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876541 HangingFrom: (identity + 3) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else
	    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	}
      else
	[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];

      [self setWorkInProgress: 0 AtSide: 3];
      if (firstLinkOfWestChain != nil)
	[firstLinkOfWestChain setWorkInProgress: 0];
      if (secondLinkOfWestChain != nil)
	[secondLinkOfWestChain setWorkInProgress: 0];
      if (thirdLinkOfWestChain != nil)
	[thirdLinkOfWestChain setWorkInProgress: 0];
      if (fourthLinkOfWestChain != nil)
	[fourthLinkOfWestChain setWorkInProgress: 0];
      if (fifthLinkOfWestChain != nil)
	[fifthLinkOfWestChain setWorkInProgress: 0];
      if (sixthLinkOfWestChain != nil)
	[sixthLinkOfWestChain setWorkInProgress: 0];
      if (seventhLinkOfWestChain != nil)
	[seventhLinkOfWestChain setWorkInProgress: 0];
      if (eighthLinkOfWestChain != nil)
	[eighthLinkOfWestChain setWorkInProgress: 0];
    }
  else
    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];


  if (workInProgressSouth == 1)
    {
      firstLinkOfSouthChain = [world getObjectAtX: xPos Y: yPosSouth];
      frontSideOfSouthChain = 4;
      if (firstLinkOfSouthChain != nil)
	[self addFirmSouth: 8 Or: 0 Or: 0 StartingFrom: firstLinkOfSouthChain];
      if (firmAdded.name == 8)
	{
	  finisherIdentity = [firmAdded.pointer getIdentity];
	  secondLinkOfSouthChain = firmAdded.pointer;
	  frontSideOfSouthChain = firmAdded.direction;
	  if (secondLinkOfSouthChain != nil)
	    {
	      if (frontSideOfSouthChain == 4)
		[self addFirmSouth: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfSouthChain];
	      else if (frontSideOfSouthChain == 3)
		[self addFirmWest: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfSouthChain];
	      else if (frontSideOfSouthChain == 1)
		[self addFirmEast: 6 Or: 7 Or: 0 StartingFrom: secondLinkOfSouthChain];
	    }
	  if (firmAdded.name == 6)
	    {
	      weaverIdentity = [firmAdded.pointer getIdentity];
	      thirdLinkOfSouthChain = firmAdded.pointer;
	      frontSideOfSouthChain = firmAdded.direction;
	      if (thirdLinkOfSouthChain != nil)
		{
		  if (frontSideOfSouthChain == 1)
		    [self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];
		  else if (frontSideOfSouthChain == 2)
		    [self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];
		  else if (frontSideOfSouthChain == 3)
		    [self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];
		  else if (frontSideOfSouthChain == 4)
		    [self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];		    
		}	      
	      if (firmAdded.name == 5)
		{
		  fourthLinkOfSouthChain = firmAdded.pointer;
		  frontSideOfSouthChain = firmAdded.direction;
		  if (fourthLinkOfSouthChain != nil)
		    {
		      if (frontSideOfSouthChain == 1)
			[self addFirmEast: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfSouthChain];
		      else if (frontSideOfSouthChain == 2)
			[self addFirmNorth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfSouthChain];
		      else if (frontSideOfSouthChain == 3)
			[self addFirmWest: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfSouthChain];
		      else if (frontSideOfSouthChain == 4)
			[self addFirmSouth: 3 Or: 4 Or: 7 StartingFrom: fourthLinkOfSouthChain];
		    }
		  if (firmAdded.name == 3)
		    {
		      fifthLinkOfSouthChain = firmAdded.pointer;
		      frontSideOfSouthChain = firmAdded.direction;
		      if (fifthLinkOfSouthChain != nil)
			{
			  if (frontSideOfSouthChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  frontSideOfSouthChain = firmAdded.direction;
			  if (sixthLinkOfSouthChain != nil)
			    {
			      if (frontSideOfSouthChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865371 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      frontSideOfSouthChain = firmAdded.direction;
			      if (seventhLinkOfSouthChain != nil)
				{
				  if (frontSideOfSouthChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfSouthChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98653721 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 4)
		    {
		      fifthLinkOfSouthChain = firmAdded.pointer;
		      frontSideOfSouthChain = firmAdded.direction;
		      if (fifthLinkOfSouthChain != nil)
			{
			  if (frontSideOfSouthChain == 1)
			    [self addFirmEast: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 2)
			    [self addFirmNorth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 3)
			    [self addFirmWest: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 4)
			    [self addFirmSouth: 7 Or: 0 Or: 0 StartingFrom: fifthLinkOfSouthChain];		    
			}
		      if (firmAdded.name == 7)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  frontSideOfSouthChain = firmAdded.direction;
			  if (sixthLinkOfSouthChain != nil)
			    {
			      if (frontSideOfSouthChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865471 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else if (firmAdded.name == 7)
		    {
		      fifthLinkOfSouthChain = firmAdded.pointer;
		      frontSideOfSouthChain = firmAdded.direction;
		      if (fifthLinkOfSouthChain != nil)
			{
			  if (frontSideOfSouthChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 986571 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  frontSideOfSouthChain = firmAdded.direction;
			  if (sixthLinkOfSouthChain != nil)
			    {
			      if (frontSideOfSouthChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865731 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      frontSideOfSouthChain = firmAdded.direction;
			      if (seventhLinkOfSouthChain != nil)
				{
				  if (frontSideOfSouthChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfSouthChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98657321 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  frontSideOfSouthChain = firmAdded.direction;
			  if (sixthLinkOfSouthChain != nil)
			    {
			      if (frontSideOfSouthChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9865741 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else if (firmAdded.name == 7)
	    {
	      thirdLinkOfSouthChain = firmAdded.pointer;
	      frontSideOfSouthChain = firmAdded.direction;
	      if (thirdLinkOfSouthChain != nil)
		{
		  if (frontSideOfSouthChain == 1)
		    [self addFirmEast: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];
		  else if (frontSideOfSouthChain == 2)
		    [self addFirmNorth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];
		  else if (frontSideOfSouthChain == 3)
		    [self addFirmWest: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];
		  else if (frontSideOfSouthChain == 4)
		    [self addFirmSouth: 6 Or: 0 Or: 0 StartingFrom: thirdLinkOfSouthChain];		    
		}	      
	      if (firmAdded.name == 6)
		{
		  weaverIdentity = [firmAdded.pointer getIdentity];
		  fourthLinkOfSouthChain = firmAdded.pointer;
		  frontSideOfSouthChain = firmAdded.direction;
		  if (fourthLinkOfSouthChain != nil)
		    {
		      if (frontSideOfSouthChain == 1)
			[self addFirmEast: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfSouthChain];
		      else if (frontSideOfSouthChain == 2)
			[self addFirmNorth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfSouthChain];
		      else if (frontSideOfSouthChain == 3)
			[self addFirmWest: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfSouthChain];
		      else if (frontSideOfSouthChain == 4)
			[self addFirmSouth: 5 Or: 0 Or: 0 StartingFrom: fourthLinkOfSouthChain];		    
		    }
		  if (firmAdded.name == 5)
		    {
		      fifthLinkOfSouthChain = firmAdded.pointer;
		      frontSideOfSouthChain = firmAdded.direction;
		      if (fifthLinkOfSouthChain != nil)
			{
			  if (frontSideOfSouthChain == 1)
			    [self addFirmEast: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 2)
			    [self addFirmNorth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 3)
			    [self addFirmWest: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];
			  else if (frontSideOfSouthChain == 4)
			    [self addFirmSouth: 1 Or: 3 Or: 4 StartingFrom: fifthLinkOfSouthChain];		    
			}
		      if (firmAdded.name == 1)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  [observerSwarm communicateChain: 987651 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			}
		      else if (firmAdded.name == 3)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  frontSideOfSouthChain = firmAdded.direction;
			  if (sixthLinkOfSouthChain != nil)
			    {
			      if (frontSideOfSouthChain == 1)
				[self addFirmEast: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 2)
				[self addFirmNorth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 3)
				[self addFirmWest: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 4)
				[self addFirmSouth: 1 Or: 2 Or: 0 StartingFrom: sixthLinkOfSouthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876531 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			    }
			  else if (firmAdded.name == 2)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      frontSideOfSouthChain = firmAdded.direction;
			      if (seventhLinkOfSouthChain != nil)
				{
				  if (frontSideOfSouthChain == 1)
				    [self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 2)
				    [self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 3)
				    [self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];
				  else if (frontSideOfSouthChain == 4)
				    [self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: seventhLinkOfSouthChain];		    
				}
			      if (firmAdded.name == 1)
				{
				  eighthLinkOfSouthChain = firmAdded.pointer;
				  [observerSwarm communicateChain: 98765321 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
				}
			      else
				[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else if (firmAdded.name == 4)
			{
			  sixthLinkOfSouthChain = firmAdded.pointer;
			  frontSideOfSouthChain = firmAdded.direction;
			  if (sixthLinkOfSouthChain != nil)
			    {
			      if (frontSideOfSouthChain == 1)
				[self addFirmEast: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 2)
				[self addFirmNorth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 3)
				[self addFirmWest: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];
			      else if (frontSideOfSouthChain == 4)
				[self addFirmSouth: 1 Or: 0 Or: 0 StartingFrom: sixthLinkOfSouthChain];		    
			    }
			  if (firmAdded.name == 1)
			    {
			      seventhLinkOfSouthChain = firmAdded.pointer;
			      [observerSwarm communicateChain: 9876541 HangingFrom: (identity + 4) Including: finisherIdentity And: weaverIdentity];
			    }
			  else
			    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
			}
		      else
			[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		    }
		  else
		    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
		}
	      else
		[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	    }
	  else
	    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];
	}
      else
	[observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];

      [self setWorkInProgress: 0 AtSide: 4];
      if (firstLinkOfSouthChain != nil)
	[firstLinkOfSouthChain setWorkInProgress: 0];
      if (secondLinkOfSouthChain != nil)
	[secondLinkOfSouthChain setWorkInProgress: 0];
      if (thirdLinkOfSouthChain != nil)
	[thirdLinkOfSouthChain setWorkInProgress: 0];
      if (fourthLinkOfSouthChain != nil)
	[fourthLinkOfSouthChain setWorkInProgress: 0];
      if (fifthLinkOfSouthChain != nil)
	[fifthLinkOfSouthChain setWorkInProgress: 0];
      if (sixthLinkOfSouthChain != nil)
	[sixthLinkOfSouthChain setWorkInProgress: 0];
      if (seventhLinkOfSouthChain != nil)
	[seventhLinkOfSouthChain setWorkInProgress: 0];
      if (eighthLinkOfSouthChain != nil)
	[eighthLinkOfSouthChain setWorkInProgress: 0];
    }
  else
    [observerSwarm communicateChain: 0 HangingFrom: 0 Including: 0 And: 0];


  // Here we empty the list of employed firms. The purpose of this list is that, during a step, -findAround does not find the same firms every time it is called by the same Middleman.

  employedFirm = self;
  while (employedFirm != nil)
    employedFirm = [employedFirmsList removeFirst];
  [employedFirmsList addFirst: nil];

  
  // Whatever Middleman did, it must be confirmed in its position.
  
  [world putObject: nil atX: xPos Y: yPos];
  [world putObject: self atX: xPos Y: yPos];

  return self;
}




// This method is only called by Middleman's -step method above, in order to add a new firm to a production chain.  In its turn, this method calls the -findAround method below.

- (struct resultOfAddFirm)addFirmEast: (int)Firm1 Or: (int)Firm2 Or: (int)Firm3 StartingFrom: (id)lastLink
{
  int freeSpace[5];
  int firmAddedToProductionChain;
  int leftOrRight;
  
  int distanceX, distanceY;
  int xToMoveTo, yToMoveTo;

  firmAddedToProductionChain = 0;
  distanceX = 1;
  distanceY = 1;
  xToMoveTo = 0;
  yToMoveTo = 0;

  firmAdded.name = 0;
  firmAdded.pointer = nil;
  firmAdded.direction = 0;

  freeSpace[0] = [lastLink doYouHaveFreeSpaceAround];
  freeSpace[1] = [lastLink doYouHaveFreeSpaceEast];
  freeSpace[2] = [lastLink doYouHaveFreeSpaceNorth];
  freeSpace[3] = 0;
  freeSpace[4] = [lastLink doYouHaveFreeSpaceSouth];

  if (freeSpace[0] == 1)
    {
      while ((distanceX <= worldXSize/2) && (distanceY <= worldYSize/2) && (firmAddedToProductionChain == 0))
	{
	  [self findAroundWho1: Firm1 Who2: Firm2 Who3: Firm3 WhereX: distanceX WhereY: distanceY];
	  if (firmFoundAround.name != 0)
	    {
	      firmAdded.name = firmFoundAround.name;
	      firmAdded.pointer = firmFoundAround.pointer;
	      if (freeSpace[1] == 1)
		{
		  xToMoveTo = [lastLink getXPosEast];
		  yToMoveTo = [lastLink getYPos];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 1;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[2] == 1) && (freeSpace[4] == 0))
		{
		  xToMoveTo = [lastLink getXPos];
		  yToMoveTo = [lastLink getYPosNorth];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 2;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[2] == 0) && (freeSpace[4] == 1))
		{
		  xToMoveTo = [lastLink getXPos];
		  yToMoveTo = [lastLink getYPosSouth];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 4;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[2] == 1) && (freeSpace[4] == 1))
		{
		  leftOrRight = [uniformIntRand getIntegerWithMin: 0 withMax: 1];
		  if (leftOrRight == 0)
		    {
		      xToMoveTo = [lastLink getXPos];
		      yToMoveTo = [lastLink getYPosNorth];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 2;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		  else if (leftOrRight == 1)
		    {
		      xToMoveTo = [lastLink getXPos];
		      yToMoveTo = [lastLink getYPosSouth];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 4;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		}
	      else
		{
		  firmAdded.name = 0;
		  firmAdded.pointer = nil;
		}
	    }
	  if (worldXSize/2 == worldYSize/2)
	    {
	      distanceX++;
	      distanceY++;
	    }
	  else if (worldXSize/2 > worldYSize/2)
	    {
	      distanceX++;
	      if (distanceY < worldYSize/2)
		distanceY++;
	    }
	  else if (worldYSize/2 > worldXSize/2)
	    {
	      if (distanceX < worldXSize/2)
		distanceX++;
	      distanceY++;
	    }
	}
    }
  
  return firmAdded;
}

- (struct resultOfAddFirm)addFirmNorth: (int)Firm1 Or: (int)Firm2 Or: (int)Firm3 StartingFrom: (id)lastLink
{
  int freeSpace[5];
  int firmAddedToProductionChain;
  int leftOrRight;
  
  int distanceX, distanceY;
  int xToMoveTo, yToMoveTo;
  
  firmAddedToProductionChain = 0;
  distanceX = 1;
  distanceY = 1;
  xToMoveTo = 0;
  yToMoveTo = 0;
  
  firmAdded.name = 0;
  firmAdded.pointer = nil;
  firmAdded.direction = 0;
  
  freeSpace[0] = [lastLink doYouHaveFreeSpaceAround];
  freeSpace[1] = [lastLink doYouHaveFreeSpaceEast];
  freeSpace[2] = [lastLink doYouHaveFreeSpaceNorth];
  freeSpace[3] = [lastLink doYouHaveFreeSpaceWest];
  freeSpace[4] = 0;

  if (freeSpace[0] == 1)
    {
      while ((distanceX <= worldXSize/2) && (distanceY <= worldYSize/2) && (firmAddedToProductionChain == 0))
	{
	  [self findAroundWho1: Firm1 Who2: Firm2 Who3: Firm3 WhereX: distanceX WhereY: distanceY];
	  if (firmFoundAround.name != 0)
	    {
	      firmAdded.name = firmFoundAround.name;
	      firmAdded.pointer = firmFoundAround.pointer;
	      if (freeSpace[2] == 1)
		{
		  xToMoveTo = [lastLink getXPos];
		  yToMoveTo = [lastLink getYPosNorth];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 2;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[3] == 1) && (freeSpace[1] == 0))
		{
		  xToMoveTo = [lastLink getXPosWest];
		  yToMoveTo = [lastLink getYPos];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 3;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[3] == 0) && (freeSpace[1] == 1))
		{
		  xToMoveTo = [lastLink getXPosEast];
		  yToMoveTo = [lastLink getYPos];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 1;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[3] == 1) && (freeSpace[1] == 1))
		{
		  leftOrRight = [uniformIntRand getIntegerWithMin: 0 withMax: 1];
		  if (leftOrRight == 0)
		    {
		      xToMoveTo = [lastLink getXPosWest];
		      yToMoveTo = [lastLink getYPos];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 3;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		  else if (leftOrRight == 1)
		    {
		      xToMoveTo = [lastLink getXPosEast];
		      yToMoveTo = [lastLink getYPos];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 1;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		}
	      else
		{
		  firmAdded.name = 0;
		  firmAdded.pointer = nil;
		}
	    }
	  if (worldXSize/2 == worldYSize/2)
	    {
	      distanceX++;
	      distanceY++;
	    }
	  else if (worldXSize/2 > worldYSize/2)
	    {
	      distanceX++;
	      if (distanceY < worldYSize/2)
		distanceY++;
	    }
	  else if (worldYSize/2 > worldXSize/2)
	    {
	      if (distanceX < worldXSize/2)
		distanceX++;
	      distanceY++;
	    }
	}
    }
  
  return firmAdded;
}

- (struct resultOfAddFirm)addFirmWest: (int)Firm1 Or: (int)Firm2 Or: (int)Firm3 StartingFrom: (id)lastLink
{
  int freeSpace[5];
  int firmAddedToProductionChain;
  int leftOrRight;
  
  int distanceX, distanceY;
  int xToMoveTo, yToMoveTo;
  
  firmAddedToProductionChain = 0;
  distanceX = 1;
  distanceY = 1;
  xToMoveTo = 0;
  yToMoveTo = 0;
  
  firmAdded.name = 0;
  firmAdded.pointer = nil;
  firmAdded.direction = 0;
  
  freeSpace[0] = [lastLink doYouHaveFreeSpaceAround];
  freeSpace[1] = 0;
  freeSpace[2] = [lastLink doYouHaveFreeSpaceNorth];
  freeSpace[3] = [lastLink doYouHaveFreeSpaceWest];
  freeSpace[4] = [lastLink doYouHaveFreeSpaceSouth];

  if (freeSpace[0] == 1)
    {
      while ((distanceX <= worldXSize/2) && (distanceY <= worldYSize/2) && (firmAddedToProductionChain == 0))
	{
	  [self findAroundWho1: Firm1 Who2: Firm2 Who3: Firm3 WhereX: distanceX WhereY: distanceY];
	  if (firmFoundAround.name != 0)
	    {
	      firmAdded.name = firmFoundAround.name;
	      firmAdded.pointer = firmFoundAround.pointer;
	      if (freeSpace[3] == 1)
		{
		  xToMoveTo = [lastLink getXPosWest];
		  yToMoveTo = [lastLink getYPos];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 3;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[4] == 1) && (freeSpace[2] == 0))
		{
		  xToMoveTo = [lastLink getXPos];
		  yToMoveTo = [lastLink getYPosSouth];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 4;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[4] == 0) && (freeSpace[2] == 1))
		{
		  xToMoveTo = [lastLink getXPos];
		  yToMoveTo = [lastLink getYPosNorth];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 2;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[4] == 1) && (freeSpace[2] == 1))
		{
		  leftOrRight = [uniformIntRand getIntegerWithMin: 0 withMax: 1];
		  if (leftOrRight == 0)
		    {
		      xToMoveTo = [lastLink getXPos];
		      yToMoveTo = [lastLink getYPosSouth];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 4;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		  else if (leftOrRight == 1)
		    {
		      xToMoveTo = [lastLink getXPos];
		      yToMoveTo = [lastLink getYPosNorth];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 2;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		}
	      else
		{
		  firmAdded.name = 0;
		  firmAdded.pointer = nil;
		}
	    }
	  if (worldXSize/2 == worldYSize/2)
	    {
	      distanceX++;
	      distanceY++;
	    }
	  else if (worldXSize/2 > worldYSize/2)
	    {
	      distanceX++;
	      if (distanceY < worldYSize/2)
		distanceY++;
	    }
	  else if (worldYSize/2 > worldXSize/2)
	    {
	      if (distanceX < worldXSize/2)
		distanceX++;
	      distanceY++;
	    }
	}
    }
  
  return firmAdded;
}

- (struct resultOfAddFirm)addFirmSouth: (int)Firm1 Or: (int)Firm2 Or: (int)Firm3 StartingFrom: (id)lastLink
{
  int freeSpace[5];
  int firmAddedToProductionChain;
  int leftOrRight;
  
  int distanceX, distanceY;
  int xToMoveTo, yToMoveTo;
  
  firmAddedToProductionChain = 0;
  distanceX = 1;
  distanceY = 1;
  xToMoveTo = 0;
  yToMoveTo = 0;
  
  firmAdded.name = 0;
  firmAdded.pointer = nil;
  firmAdded.direction = 0;
  
  freeSpace[0] = [lastLink doYouHaveFreeSpaceAround];
  freeSpace[1] = [lastLink doYouHaveFreeSpaceEast];
  freeSpace[2] = 0;
  freeSpace[3] = [lastLink doYouHaveFreeSpaceWest];
  freeSpace[4] = [lastLink doYouHaveFreeSpaceSouth];

  if (freeSpace[0] == 1)
    {
      while ((distanceX <= worldXSize/2) && (distanceY <= worldYSize/2) && (firmAddedToProductionChain == 0))
	{
	  [self findAroundWho1: Firm1 Who2: Firm2 Who3: Firm3 WhereX: distanceX WhereY: distanceY];
	  if (firmFoundAround.name != 0)
	    {
	      firmAdded.name = firmFoundAround.name;
	      firmAdded.pointer = firmFoundAround.pointer;
	      if (freeSpace[4] == 1)
		{
		  xToMoveTo = [lastLink getXPos];
		  yToMoveTo = [lastLink getYPosSouth];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 4;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[3] == 1) && (freeSpace[1] == 0))
		{
		  xToMoveTo = [lastLink getXPosWest];
		  yToMoveTo = [lastLink getYPos];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 3;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[3] == 0) && (freeSpace[1] == 1))
		{
		  xToMoveTo = [lastLink getXPosEast];
		  yToMoveTo = [lastLink getYPos];
		  [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		  firmAdded.direction = 1;
		  [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		  [firmAdded.pointer setWorkInProgress: 1];
		  firmAddedToProductionChain = 1;
		}
	      else if ((freeSpace[3] == 1) && (freeSpace[1] == 1))
		{
		  leftOrRight = [uniformIntRand getIntegerWithMin: 0 withMax: 1];
		  if (leftOrRight == 0)
		    {
		      xToMoveTo = [lastLink getXPosWest];
		      yToMoveTo = [lastLink getYPos];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 3;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		  else if (leftOrRight == 1)
		    {
		      xToMoveTo = [lastLink getXPosEast];
		      yToMoveTo = [lastLink getYPos];
		      [modelSwarm bookX: xToMoveTo Y: yToMoveTo];
		      firmAdded.direction = 1;
		      [firmAdded.pointer setX: xToMoveTo Y: yToMoveTo];
		      [firmAdded.pointer setWorkInProgress: 1];
		      firmAddedToProductionChain = 1;
		    }
		}
	      else
		{
		  firmAdded.name = 0;
		  firmAdded.pointer = nil;
		}
	    }
	  if (worldXSize/2 == worldYSize/2)
	    {
	      distanceX++;
	      distanceY++;
	    }
	  else if (worldXSize/2 > worldYSize/2)
	    {
	      distanceX++;
	      if (distanceY < worldYSize/2)
		distanceY++;
	    }
	  else if (worldYSize/2 > worldXSize/2)
	    {
	      if (distanceX < worldXSize/2)
		distanceX++;
	      distanceY++;
	    }
	}
    }
  
  return firmAdded;
}




// This method is only called by Middleman's -addFirm methods above.  It explores the space along a rectangle centered around Middleman, searching for an unemployed firm of the required kind. Method -findAround must be applied iteratively from Middleman outwards, in order to look for the nearest firm.  Arguments whoYouWant1, whoYouWant2 and whoYouWant3 represent the firms this method must look for.  Argument xDistance represents the distance of the rectangle from Middleman's xPos, and it is equal to half of the length of the rectangle along the x axis.  Argument yDistance represents the distance of the rectangle from Middleman's yPos, and it is equal to half of the length of the rectangle along the y axis.  The method first analyzes the lower horizontal side of the rectangle, then the upper horizontal side, then the left vertical side, and finally the right vertical side. Each side is eventually split into two segments if it extends out of the borders of the raster; for this reason, two minima and two maxima are used both for x and for y.  Both xDistance and yDistance must not exceed a half of the corresponding raster dimensions. If this method is called several times during the same step, it yields a different firm every time. In order to do this, it needs to store previously found firms in a list and to check the identity of each newly found firm with the identities of the previously found ones.

- (struct resultOfFindAround)findAroundWho1: (int)whoYouWant1 Who2: (int)whoYouWant2 Who3: (int)whoYouWant3 WhereX: (int)xDistance WhereY: (int)yDistance
{
  int x, xMin1, xMin2, xMax1, xMax2;
  int y, yMin1, yMin2, yMax1, yMax2;
  id object;
  int employed;
  int Iam;
  int myNameIs;
  int found = 0;

  firmFoundAround.name = 0;
  firmFoundAround.pointer = nil;
  
  
  if ((xPos - xDistance) < 0)
    {
      xMin1 = 0;
      xMax1 = (xPos + xDistance);
      xMin2 = (worldXSize + xPos - xDistance);
      xMax2 = (worldXSize - 1);
    }
  else if ((xPos + xDistance) > (worldXSize - 1))
    {
      xMin1 = (xPos - xDistance);
      xMax1 = (worldXSize - 1);
      xMin2 = 0;
      xMax2 = (xPos + xDistance - worldXSize);
    }
  else
    {
      xMin1 = (xPos - xDistance);
      xMax1 = (xPos + xDistance);
      xMin2 = xPos;
      xMax2 = xPos;
    }
  
  if ((yPos - yDistance) < 0)
    y = (worldYSize + yPos - yDistance);
  else
    y = (yPos - yDistance);
  x = xMin1;
  while ((x <= xMax1) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      x++;
    }
  x = xMin2;
  while ((x <= xMax2) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      x++;
    }
  
  if ((yPos + yDistance) > (worldYSize - 1))
    y = (yPos + yDistance - worldYSize);
  else
    y = (yPos + yDistance);
  x = xMin1;
  while ((x <= xMax1) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      x++;
    }
  x = xMin2;
  while ((x <= xMax2) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      x++;
    }
  
  
  if ((yPos - yDistance) < 0)
    {
      yMin1 = 0;
      yMax1 = (yPos + yDistance);
      yMin2 = (worldYSize + yPos - yDistance);
      yMax2 = (worldYSize - 1);
    }
  else if ((yPos + yDistance) > (worldYSize - 1))
    {
      yMin1 = (yPos - yDistance);
      yMax1 = (worldYSize - 1);
      yMin2 = 0;
      yMax2 = (yPos + yDistance - worldYSize);
    }
  else
    {
      yMin1 = (yPos - yDistance);
      yMax1 = (yPos + yDistance);
      yMin2 = yPos;
      yMax2 = yPos;
    }
  
  if ((xPos - xDistance) < 0)
    x = (worldXSize + xPos - xDistance);
  else
    x = (xPos - xDistance);
  y = yMin1;
  while ((y <= yMax1) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      y++;
    }
  y = yMin2;
  while ((y <= yMax2) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      y++;
    }
  
  if ((xPos + xDistance) > (worldXSize - 1))
    x = (xPos + xDistance - worldXSize);
  else
    x = (xPos + xDistance);
  y = yMin1;
  while ((y <= yMax1) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      y++;
    }
  y = yMin2;
  while ((y <= yMax2) && (found == 0))
    {
      object = [world getObjectAtX: x Y: y];
      if ((object != nil) && ([object areYouWorking][0] == 0))
	{
	  Iam = [object whoAreYou];
	  myNameIs = [object getIdentity];
	  if ((Iam == whoYouWant1) || (Iam == whoYouWant2) || (Iam == whoYouWant3))
	    {
	      employed = 0;
	      employedFirm = self;
	      while (employedFirm != nil)
		{
		  employedFirm = [employedFirmsList removeFirst];
		  if (employedFirm != nil)
		    {
		      employedFirmIs = [employedFirm whoAreYou];
		      employedFirmsName = [employedFirm getIdentity];
		      if ((Iam == employedFirmIs) && (myNameIs == employedFirmsName))
			employed = 1;
		    }
		  [employedFirmsList addLast: employedFirm];
		}
	      if (employed == 0)
		{
		  found = 1;
		  [employedFirmsList addFirst: object];
		  firmFoundAround.name = Iam;
		  firmFoundAround.pointer = object;
		}
	    }
	}
      y++;
    }
  
  
  return firmFoundAround;
}




// This method draws a Middleman on the raster.

- drawSelfOn: (id <Raster>)R
{
  [R drawPointX: xPos Y: yPos Color: 10];
  return self;
}




// This method drops the list of firms employed in production chains during one step. It is called by -firmsCreateAndDestroy of ModelSwarm.

- dropObjects
{
  [employedFirmsList drop];
  return self;
}




@end  

































