// ModelSwarm.m

//Copyright (C) 2001 by Guido Fioretti, via Angeloni 11, I - 05100 Terni. This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

#import "ModelSwarm.h"
#import "ObserverSwarm.h"
#import <random.h>
#import <activity.h>






@implementation ModelSwarm  




// This method reads the number of firms from inputMatrix. Object inputMatrix is created in -builObjects either from FirmsNumber1:1.txt or from FirmsNumber1:2.txt or from FirmsNumber1:10.txt, according to the value of the global variable modelScale. Every time it is called, it reads one line and jumps down to the next one. The last line of FirmsNumber1:*.txt must be made by zeroes in order to tell ModelSwarm that data are finished.

- importFirmsNumber: (int)year
{
  char inputRow[100];
  if([inputMatrix getLine: inputRow] != 0)
    {
      sscanf(inputRow,"%i %i %i %i %i %i %i %i %i %i %i", &traderRawMaterialsNumber, &ragsCollectorNumber, &carderSpinningNumber, &comberSpinningNumber, &warperNumber, &weaverNumber, &dyeingPlantNumber, &finisherNumber, &middlemanNumber, &traderFinishedProductsNumber, &totalFirmsNumber);
      [inputMatrix skipLine];
    }
  return self;
}




// This method enables ModelSwarm to see ObserverSwarm.  This is necessary in order to pass on to the raster a new title at every new year, as well as to stop the simulation if parameters are not correct or once data are finished.  Method -setObserverSwarm is activated from ObserverSwarm.  In order to enable ModelSwarm and ObserverSwarm to see each other without causing cross-references, ObserverSwarm.h is imported in ModelSwarm.m and ModelSwarm.h is imported in ObserverSwarm.h  Object observerSwarm is declared in ModelSwarm.h.

- setObserverSwarm: OS
{
  observerSwarm = OS;
  return self;
}




// The following methods allow the ObserverSwarm to see the lists of firms.

- getTraderRawMaterialsList
{
  return traderRawMaterialsList;
}

- getRagsCollectorList
{
  return ragsCollectorList;
}

- getCarderSpinningList
{
  return carderSpinningList;
}

- getComberSpinningList
{
  return comberSpinningList;
}

- getWarperList
{
  return warperList;
}

- getWeaverList
{
  return weaverList;
}

- getDyeingPlantList
{
  return dyeingPlantList;
}

- getFinisherList
{
  return finisherList;
}

- getMiddlemanList
{
  return middlemanList;
}

- getTraderFinishedProductsList
{
  return traderFinishedProductsList;
}




// This method allows ObserverSwarm to see the grid where firms are.

- getWorld
{
  return world;
}




// This method initializes the global variables that appear in the probe: modelScale, worldXSize, worldYSize, xWatchRange, yWatchRange and variance.

+ createBegin: aZone
{
  ModelSwarm *obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];
  obj->modelScale = 10;
  obj->worldXSize = 150;
  obj->worldYSize = 200;
  obj->xWatchRange = 10;
  obj->yWatchRange = 10;
  obj->variance = 10.0;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "modelScale"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldXSize"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldYSize"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "xWatchRange"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "yWatchRange"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "variance"
				    inClass: [self class]]];
  [probeLibrary setProbeMap: probeMap For: [self class]]; 

  return obj;
}




// This method initializes the global variables that do not appear in the probe: iW, iF, iM, stepCounter, yearCounter, the number of firms and the bookedPositions matrix. Counters iW, iF and iM are only used by -firmsCreateAndDestroy, but they cannot be initialized there since that method is called every year.

- createEnd
{
  stepCounter = 0;
  yearCounter = 1946;

  iTRM = 1;
  iRC = 1;
  iCaS = 1;
  iCoS = 1;
  iWa = 1;
  iWe = 1;
  iDP = 1;
  iF = 1;
  iTFP = 1;
  iM = 0;

  traderRawMaterialsNumber = 0;
  ragsCollectorNumber = 0;
  carderSpinningNumber = 0;
  comberSpinningNumber = 0;
  warperNumber = 0;
  weaverNumber = 0;
  dyeingPlantNumber = 0;
  finisherNumber = 0;
  middlemanNumber = 0;
  traderFinishedProductsNumber = 0;

  for (p=0; p<1000; p++)
    for (p=0; p<1000; p++)
      bookedPositions[p][q] = 0;
  
  return [super createEnd];
}




// This method is activated by -buildObjects of ObserverSwarm. First of all, it writes warning messages and stops the simulation if parameter values are illegal. Then it proceeds to create an input matrix. Subsequently, it creates a grid "world" to represent the positions of all firms and communicates its dimensions to ObserverSwarm. Finally, it creates a list for each kind of firm, as well as a list shuffler.

- buildObjects
{
  if (worldXSize > 1000)
    {
      printf("WARNING: worldXSize cannot exceed 1000!\n");
      [observerSwarm stop];
    }
  else if (worldXSize < 10)
    {
      printf("WARNING: worldXSize cannot be less than 10!\n");
      [observerSwarm stop];
    }
  if (worldYSize > 1000)
    {
      printf("WARNING: worldYSize cannot exceed 1000!\n");
      [observerSwarm stop];
    }
  else if (worldYSize < 10)
    {
      printf("WARNING: worldYSize cannot be less than 10!\n");
      [observerSwarm stop];
    }
  if (xWatchRange > worldXSize)
    {
      printf("WARNING: xWatchRange cannot exceed worldXSize!\n");
      [observerSwarm stop];
    }
  else if (xWatchRange < 1)
    {
      printf("WARNING: xWatchRange cannot be less than 1!\n");
      [observerSwarm stop];
    }
  if (yWatchRange > worldYSize)
    {
      printf("WARNING: yWatchRange cannot exceed worldYSize!\n");
      [observerSwarm stop];
    }
  else if (yWatchRange < 1)
    {
      printf("WARNING: yWatchRange cannot be less than 1!\n");
      [observerSwarm stop];
    }

  if (modelScale == 1)
    inputMatrix = [InFile create: [self getZone] withName: "FirmsNumberScale1to1.txt"];
  else if (modelScale == 2)
    inputMatrix = [InFile create: [self getZone] withName: "FirmsNumberScale1to2.txt"];
  else if (modelScale == 10)
    inputMatrix = [InFile create: [self getZone] withName: "FirmsNumberScale1to10.txt"];
  else
    {
      printf("WARNING: This model supports 1:1, 1:2 and 1:10 scaling only!\n");
      [observerSwarm stop];
    }

  world = [Grid2d createBegin: self];
  [world setSizeX: worldXSize Y: worldYSize];
  world = [world createEnd];
  [world fillWithObject: nil];

  [observerSwarm setRasterXSize: worldXSize];
  [observerSwarm setRasterYSize: worldYSize];

  traderRawMaterialsList = [List create: self];
  ragsCollectorList = [List create: self];
  carderSpinningList = [List create: self];
  comberSpinningList = [List create: self];
  warperList = [List create: self];
  weaverList = [List create: self];
  dyeingPlantList = [List create: self];
  finisherList = [List create: self];
  middlemanList = [List create: self];
  traderFinishedProductsList = [List create: self];

  shuffler = [ListShuffler create: [self getZone]];

  return self;
}




// This method is activated by -buildActions of ObserverSwarm. It creates a list of simulation actions, as well as a schedule that executes the modelActions.

- buildActions
{
  
  modelActions = [ActionGroup create: self];
  [modelActions createActionTo: self message: M(firmsCreateAndDestroy)];
  [modelActions createActionForEach: traderRawMaterialsList message: M(step)];
  [modelActions createActionForEach: ragsCollectorList message: M(step)];
  [modelActions createActionForEach: carderSpinningList message: M(step)];
  [modelActions createActionForEach: comberSpinningList message: M(step)];
  [modelActions createActionForEach: warperList message: M(step)];
  [modelActions createActionForEach: weaverList message: M(step)];
  [modelActions createActionForEach: dyeingPlantList message: M(step)];
  [modelActions createActionForEach: finisherList message: M(step)];
  [modelActions createActionTo: self message: M(clearBookedPositions)];
  [modelActions createActionForEach: traderFinishedProductsList message: M(step)];
  [modelActions createActionForEach: middlemanList message: M(step)];
  
  modelSchedule = [Schedule createBegin: self];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createAction: modelActions]; 
  
  return self;
  
}




// Here, we activate the swarm in the context passed in. Then we activate our schedule in ourselves
  
- activateIn: swarmContext
{
  [super activateIn: swarmContext];
  [modelSchedule activateIn: self];
  return [self getSwarmActivity];
}




// At the beginning of every year, this method creates and destroyes firms according to data entailed in FirmsNumber matrices.  It stops to operate when data are no longer available in FirmsNumber, so inserting data concerning recent years would not require to change the code.  As soon as data are no longer available in FirmsNumber, it sends a message to ObserverSwarm in order to end the simulation.  Firms are destroyed after reshuffling their list.  Firms are created as follows. A position is sampled on the grid by means of a uniform random distribution. If there is nothing in that position, and if its 12 neighboring positions (east, north, west, south, northEast, northWest, southWest, southEast, eastEast, northNorth, westWest and southSouth) are empty as well, this position is communicated to the firm.  Later, each firm will put itself on the grid. This procedure is repeated until the desired number of firms has been created.  During the creating phase this method communicates to the firm its coordinates, a pointer to grid2d "world", and a pointer to ModelSwarm. The pointer to ModelSwarm is needed by all firms in order to know which positions have been booked by Middleman in order to construct a production chain. Matrix bookedPositions is defined as a [1000]X[1000] matrix, and it is initialized with this size in -createEnd. However, in order to save time, -firmsCreateAndDestroy only updates its [worldXSize]X[worldYSize] section, at the beginning of each step.   Furthermore, this method chooses identification numbers that are specific to each instance of each firm. Identification numbers are just the sequence {1,2,3,...}, except for Middleman. Middleman is assigned the sequence {0,4,8,...} in oder to allow summation to the identification numbers of its sides.  As far as it concerns TraderFinishedProducts, this method communicates the variance of the Gaussian distribution that regulates their random walk, as well as the size of their watch range when they look for a Middleman.  Finally, this method tells ObserverSwarm at which step the simulation is, in which year, and how many steps there are in one year. The number of steps is such that the product of the number of steps during one year and the total number of firms, is a constant. If the total number of firms is too large for the world size, this method writes a warning message on the screen and stops the simulation. The threshold is calculated as follows: each firms occupies its own position plus the neighboring 12 positions, since other firms are not allowed to settle down there. Firms need at least one other place to stay besides the one they occupy, in order for dynamics to take place. Thus, 13X(totalFirmsNumber) must not exceed ((worldXSize)X(worldYSize))/2.

- firmsCreateAndDestroy
{
  int x, y;
  int xEast, xWest;
  int yNorth, ySouth;
  int xEastEast, xWestWest;
  int yNorthNorth, ySouthSouth;

  int xPosDrop, yPosDrop;
  id firmToDrop;

  if (stepCounter == 0)
    {
      traderRawMaterialsOldNumber = traderRawMaterialsNumber;
      ragsCollectorOldNumber = ragsCollectorNumber;
      carderSpinningOldNumber = carderSpinningNumber;
      comberSpinningOldNumber = comberSpinningNumber;
      warperOldNumber = warperNumber;
      weaverOldNumber = weaverNumber;
      dyeingPlantOldNumber = dyeingPlantNumber;
      finisherOldNumber = finisherNumber;
      middlemanOldNumber = middlemanNumber;
      traderFinishedProductsOldNumber = traderFinishedProductsNumber;

      [self importFirmsNumber: yearCounter];
      stepsInAYear = (int)((10000000.0/(float)totalFirmsNumber)/(float)modelScale);
      [observerSwarm setStepsInAYear: stepsInAYear];
      if ((13 * totalFirmsNumber) > ((worldXSize * worldYSize) / 2))
	{
	  printf("WARNING: The world is too small for so many firms!\n");
	  [observerSwarm stop];
	}
      else
	{
	  if (totalFirmsNumber != 0)
	    {
	      [observerSwarm setYearCounter: yearCounter];
	      yearCounter++;
	      
	      if (traderRawMaterialsNumber < traderRawMaterialsOldNumber)
		{
		  [shuffler shuffleWholeList: traderRawMaterialsList];
		  for (j=1; j <= (traderRawMaterialsOldNumber - traderRawMaterialsNumber); j++)
		    {
		      firmToDrop = [traderRawMaterialsList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (traderRawMaterialsNumber > traderRawMaterialsOldNumber)
		{
		  j=1;
		  while (j <= (traderRawMaterialsNumber - traderRawMaterialsOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aTraderRawMaterials = [TraderRawMaterials createBegin: self];
			  [aTraderRawMaterials setWorld: world];
			  [aTraderRawMaterials setX: x Y: y];
			  [aTraderRawMaterials setIdentity: iTRM];
			  [aTraderRawMaterials setModelSwarm: self];
			  aTraderRawMaterials = [aTraderRawMaterials createEnd];
			  [traderRawMaterialsList addLast: aTraderRawMaterials];
			  iTRM++;
			  j++;
			}
		    }
		}
	      
	      if (ragsCollectorNumber < ragsCollectorOldNumber)
		{
		  [shuffler shuffleWholeList: ragsCollectorList];
		  for (j=1; j <= (ragsCollectorOldNumber - ragsCollectorNumber); j++)
		    {
		      firmToDrop = [ragsCollectorList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (ragsCollectorNumber > ragsCollectorOldNumber)
		{
		  j=1;
		  while (j <= (ragsCollectorNumber - ragsCollectorOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aRagsCollector = [RagsCollector createBegin: self];
			  [aRagsCollector setWorld: world];
			  [aRagsCollector setX: x Y: y];
			  [aRagsCollector setIdentity: iRC];
			  [aRagsCollector setModelSwarm: self];
			  aRagsCollector = [aRagsCollector createEnd];
			  [ragsCollectorList addLast: aRagsCollector];
			  iRC++;
			  j++;
			}
		    }
		}
	      
	      if (carderSpinningNumber < carderSpinningOldNumber)
		{
		  [shuffler shuffleWholeList: carderSpinningList];
		  for (j=1; j <= (carderSpinningOldNumber - carderSpinningNumber); j++)
		    {
		      firmToDrop = [carderSpinningList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (carderSpinningNumber > carderSpinningOldNumber)
		{
		  j=1;
		  while (j <= (carderSpinningNumber - carderSpinningOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aCarderSpinning = [CarderSpinning createBegin: self];
			  [aCarderSpinning setWorld: world];
			  [aCarderSpinning setX: x Y: y];
			  [aCarderSpinning setIdentity: iCaS];
			  [aCarderSpinning setModelSwarm: self];
			  aCarderSpinning = [aCarderSpinning createEnd];
			  [carderSpinningList addLast: aCarderSpinning];
			  iCaS++;
			  j++;
			}
		    }
		}
	      
	      if (comberSpinningNumber < comberSpinningOldNumber)
		{
		  [shuffler shuffleWholeList: comberSpinningList];
		  for (j=1; j <= (comberSpinningOldNumber - comberSpinningNumber); j++)
		    {
		      firmToDrop = [comberSpinningList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (comberSpinningNumber > comberSpinningOldNumber)
		{
		  j=1;
		  while (j <= (comberSpinningNumber - comberSpinningOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aComberSpinning = [ComberSpinning createBegin: self];
			  [aComberSpinning setWorld: world];
			  [aComberSpinning setX: x Y: y];
			  [aComberSpinning setIdentity: iCoS];
			  [aComberSpinning setModelSwarm: self];
			  aComberSpinning = [aComberSpinning createEnd];
			  [comberSpinningList addLast: aComberSpinning];
			  iCoS++;
			  j++;
			}
		    }
		}
	      
	      if (warperNumber < warperOldNumber)
		{
		  [shuffler shuffleWholeList: warperList];
		  for (j=1; j <= (warperOldNumber - warperNumber); j++)
		    {
		      firmToDrop = [warperList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (warperNumber > warperOldNumber)
		{
		  j=1;
		  while (j <= (warperNumber - warperOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aWarper = [Warper createBegin: self];
			  [aWarper setWorld: world];
			  [aWarper setX: x Y: y];
			  [aWarper setIdentity: iWa];
			  [aWarper setModelSwarm: self];
			  aWarper = [aWarper createEnd];
			  [warperList addLast: aWarper];
			  iWa++;
			  j++;
			}
		    }
		}
	      
	      if (weaverNumber < weaverOldNumber)
		{
		  [shuffler shuffleWholeList: weaverList];
		  for (j=1; j <= (weaverOldNumber - weaverNumber); j++)
		    {
		      firmToDrop = [weaverList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (weaverNumber > weaverOldNumber)
		{
		  j=1;
		  while (j <= (weaverNumber - weaverOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aWeaver = [Weaver createBegin: self];
			  [aWeaver setWorld: world];
			  [aWeaver setX: x Y: y];
			  [aWeaver setIdentity: iWe];
			  [aWeaver setModelSwarm: self];
			  aWeaver = [aWeaver createEnd];
			  [weaverList addLast: aWeaver];
			  iWe++;
			  j++;
			}
		    }
		}
	      
	      if (dyeingPlantNumber < dyeingPlantOldNumber)
		{
		  [shuffler shuffleWholeList: dyeingPlantList];
		  for (j=1; j <= (dyeingPlantOldNumber - dyeingPlantNumber); j++)
		    {
		      firmToDrop = [dyeingPlantList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (dyeingPlantNumber > dyeingPlantOldNumber)
		{
		  j=1;
		  while (j <= (dyeingPlantNumber - dyeingPlantOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aDyeingPlant = [DyeingPlant createBegin: self];
			  [aDyeingPlant setWorld: world];
			  [aDyeingPlant setX: x Y: y];
			  [aDyeingPlant setIdentity: iDP];
			  [aDyeingPlant setModelSwarm: self];
			  aDyeingPlant = [aDyeingPlant createEnd];
			  [dyeingPlantList addLast: aDyeingPlant];
			  iDP++;
			  j++;
			}
		    }
		}
	      
	      if (finisherNumber < finisherOldNumber)
		{
		  [shuffler shuffleWholeList: finisherList];
		  for (j=1; j <= (finisherOldNumber - finisherNumber); j++)
		    {
		      firmToDrop = [finisherList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop drop];
		    }
		}
	      else if (finisherNumber > finisherOldNumber)
		{
		  j=1;
		  while (j <= (finisherNumber - finisherOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aFinisher = [Finisher createBegin: self];
			  [aFinisher setWorld: world];
			  [aFinisher setX: x Y: y];
			  [aFinisher setIdentity: iF];
			  [aFinisher setModelSwarm: self];
			  aFinisher = [aFinisher createEnd];
			  [finisherList addLast: aFinisher];
			  iF++;
			  j++;
			}
		    }
		}
	      
	      if (traderFinishedProductsNumber < traderFinishedProductsOldNumber)
		{
		  [shuffler shuffleWholeList: traderFinishedProductsList];
		  for (j=1; j <= (traderFinishedProductsOldNumber - traderFinishedProductsNumber); j++)
		    {
		      firmToDrop = [traderFinishedProductsList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop dropObjects];
		      [firmToDrop drop];
		    }
		}
	      else if (traderFinishedProductsNumber > traderFinishedProductsOldNumber)
		{
		  j=1;
		  while (j <= (traderFinishedProductsNumber - traderFinishedProductsOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aTraderFinishedProducts = [TraderFinishedProducts createBegin: self];
			  [aTraderFinishedProducts setWorld: world];
			  [aTraderFinishedProducts setX: x Y: y];
			  [aTraderFinishedProducts setIdentity: iTFP];
			  [aTraderFinishedProducts setXwatchRange: xWatchRange];
			  [aTraderFinishedProducts setYwatchRange: yWatchRange];
			  [aTraderFinishedProducts setVariance: variance];
			  [aTraderFinishedProducts setModelSwarm: self];
			  [aTraderFinishedProducts buildObjects];
			  aTraderFinishedProducts = [aTraderFinishedProducts createEnd];
			  [traderFinishedProductsList addLast: aTraderFinishedProducts];
			  iTFP++;
			  j++;
			}
		    }
		}
	      
	      if (middlemanNumber < middlemanOldNumber)
		{
		  [shuffler shuffleWholeList: middlemanList];
		  for (j=1; j <= (middlemanOldNumber - middlemanNumber); j++)
		    {
		      firmToDrop = [middlemanList removeLast];
		      xPosDrop = [firmToDrop getXPos];
		      yPosDrop = [firmToDrop getYPos];
		      [world putObject: nil atX: xPosDrop Y: yPosDrop];
		      [firmToDrop dropObjects];
		      [firmToDrop drop];
		    }
		}
	      else if (middlemanNumber > middlemanOldNumber)
		{
		  j=1;
		  while (j <= (middlemanNumber - middlemanOldNumber))
		    {
		      x = [uniformIntRand getIntegerWithMin: 0 withMax: (worldXSize-1)];
		      y = [uniformIntRand getIntegerWithMin: 0 withMax: (worldYSize-1)];
		      xEast = (x + 1 + worldXSize) % worldXSize;
		      yNorth = (y + 1 + worldYSize) % worldYSize;
		      xWest = (x - 1 + worldXSize) % worldXSize;
		      ySouth = (y - 1 + worldYSize) % worldYSize;
		      xEastEast = (xEast + 1 + worldXSize) % worldXSize;
		      yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
		      xWestWest = (xWest - 1 + worldXSize) % worldXSize;
		      ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
		      if (([world getObjectAtX: x Y: y] == nil) && ([world getObjectAtX: xEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: yNorth] == nil) && ([world getObjectAtX: xWest Y: ySouth] == nil) && ([world getObjectAtX: xEast Y: ySouth] == nil) && ([world getObjectAtX: xEastEast Y: y] == nil) && ([world getObjectAtX: x Y: yNorthNorth] == nil) && ([world getObjectAtX: xWestWest Y: y] == nil) && ([world getObjectAtX: x Y: ySouthSouth] == nil))
			{
			  aMiddleman = [Middleman createBegin: self];
			  [aMiddleman setWorld: world];
			  [aMiddleman setX: x Y: y];
			  [aMiddleman setIdentity: iM];
			  [aMiddleman setModelSwarm: self];
			  [aMiddleman setObserverSwarm: observerSwarm];
			  [aMiddleman buildObjects];
			  aMiddleman = [aMiddleman createEnd];
			  [middlemanList addLast: aMiddleman];
			  iM+=4;
			  j++;
			}
		    }
		}
	    }
	  else
	    {
	      [observerSwarm setYearCounter: yearCounter];
	      [observerSwarm end];
              printf("The End\n");
	    }
	}
    }
  [observerSwarm setStepCounter: stepCounter];

  stepCounter++;
  if (stepCounter == stepsInAYear)
    stepCounter = 0;

  return self;
}




// It may happen that, during one time step, firms that are not called to be part of a production chain jump into the place where Middleman planned to construct one. This can happen because firms lists may be such that before the firms that are called to be part of a production chain move, other firms jumped already. In order to avoid this, -addFirmEast, -addFirmNorth, -addFirmWest and -addFirmSouth of Middleman communicate to ModelSwarm the positions to book for constructing a chain. This is done by method -book below, which puts a 1 in the bookedPositions matrix if a position is booked. Furthermore, it adds 12 neighboring positions to the one originally booked. Before jumping, firms ask ModelSwarm whether the position they chose was among the booked ones. This is done by method -isBooked below. After all firms jumped, booked positions are cleared by method -clearBookedPositions below, which is activated by -buildActions.

- bookX: (int)x Y: (int)y
{
  int xEast = (x + 1 + worldXSize) % worldXSize;
  int yNorth = (y + 1 + worldYSize) % worldYSize;
  int xWest = (x - 1 + worldXSize) % worldXSize;
  int ySouth = (y - 1 + worldYSize) % worldYSize;
  int xEastEast = (xEast + 1 + worldXSize) % worldXSize;
  int yNorthNorth = (yNorth + 1 + worldYSize) % worldYSize;
  int xWestWest = (xWest - 1 + worldXSize) % worldXSize;
  int ySouthSouth = (ySouth - 1 + worldYSize) % worldYSize;
 
  bookedPositions[x][y] = 1;
  bookedPositions[xEast][y] = 1;
  bookedPositions[x][yNorth] = 1;
  bookedPositions[xWest][y] = 1;
  bookedPositions[x][ySouth] = 1;
  bookedPositions[xEast][yNorth] = 1;
  bookedPositions[xWest][yNorth] = 1;
  bookedPositions[xWest][ySouth] = 1;
  bookedPositions[xEast][ySouth] = 1;
  bookedPositions[xEastEast][y] = 1;
  bookedPositions[x][yNorthNorth] = 1;
  bookedPositions[xWestWest][y] = 1;
  bookedPositions[x][ySouthSouth] = 1;
  
  return self;
}

- (int)isBookedX: (int)x Y: (int)y
{
  int yesOrNo = bookedPositions[x][y];
  return yesOrNo;
}

- clearBookedPositions
{
  p = 0;
  while (p < worldXSize)
    {
      q = 0;
      while (q < worldYSize)
	{
	  bookedPositions[p][q] = 0;
	  q++;
	}
      p++;
    }
  return self;
}




@end




















