// ObserverSwarm.m

//Copyright (C) 2001 by Guido Fioretti, via Angeloni 11, I - 05100 Terni. This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

#import "ObserverSwarm.h"
#import <simtoolsgui.h>





@implementation ObserverSwarm




// This method initializes the global variables that appear in the probe: zoomFactor and displayFrequency.

+ createBegin: aZone
{
  ObserverSwarm *obj;
  id <ProbeMap> probeMap;
  obj = [super createBegin: aZone];

  obj->zoomFactor = 5;
  obj->displayFrequency = 1;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "zoomFactor"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];
  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  return obj;
}




// This method initializes all the global variable that do not appear in the probe.

- createEnd
{
  int i;
  for (i=0; i<1000; i++)
    {
      chainDuringOneStep[i] = 0;
      chainDuringOneStepOld[i] = 0;
      hookDuringOneStep[i] = 0;
      hookDuringOneStepOld[i] = 0;
      whichFinisherDuringOneYear[i] = 0;
      whichWeaverDuringOneYear[i] = 0;
      hookDuringOneYear[i] = 0;
    }
  
  stepCounter = 0;
  oldStepCounter = 0;
  yearCounter = 1946;
  oldYearCounter = 1946;

  stepsInThisYear = 0;
  stepsInTheNextYear = 0;

  chain9865371Counter = 0;
  chain98653721Counter = 0;
  chain9865471Counter = 0;
  chain986571Counter = 0;
  chain9865731Counter = 0;
  chain98657321Counter = 0;
  chain9865741Counter = 0;
  chain987651Counter = 0;
  chain9876531Counter = 0;
  chain98765321Counter = 0;
  chain9876541Counter = 0;
  anyChainDuringOneStepCounter = 0;
  anyChainDuringOneYearCounter = 0;
  anyChainInTheBufferCounter = 0;

  chain9865371Parallelism = 0;
  chain98653721Parallelism = 0;
  chain9865471Parallelism = 0;
  chain986571Parallelism = 0;
  chain9865731Parallelism = 0;
  chain98657321Parallelism = 0;
  chain9865741Parallelism = 0;
  chain987651Parallelism = 0;
  chain9876531Parallelism = 0;
  chain98765321Parallelism = 0;
  chain9876541Parallelism = 0;

  degreeOfParallelism = 0.0;
  summedDegreeOfParallelism = 0.0;
  averagedDegreeOfParallelism = 0.0;

  degreeOfVariability = 0.0;
  summedDegreeOfVariability = 0.0;
  averagedDegreeOfVariability = 0.0;

  finisherPersistence = 0;
  summedFinisherPersistence = 0;
  degreeOfFinisherMobility = 0.0;
  weaverPersistence = 0;
  summedWeaverPersistence = 0;
  degreeOfWeaverMobility = 0.0;

  theEnd = 0;
  return [super createEnd];
}




// These methods allows ModelSwarm to communicate the world size to ObserverSwarm. Later on, ObserverSwarm builds a raster of the same size as the world's.

- setRasterXSize: (int)worldXSize
{
  rasterXSize = worldXSize;
  return self;
}

- setRasterYSize: (int)worldYSize
{
  rasterYSize = worldYSize;
  return self;
}




// This method is activated by main. This method creates the ModelSwarm, tells the control panel to wait for somebody to press the start button, tells the ModelSwarm to build its objects and communicates the existence of ObserverSwarm to -setObserverSwarm in ModelSwarm. It creates a color map, a raster and 10 displays, one for each kind of firm in the district. Furthermore, it creates two graphs and depicts two elements on each of them. Instructions in capital letters enable the "Save" button, which saves the positions of the windows on the screen.

- buildObjects
{
  [super buildObjects];

  modelSwarm = [ModelSwarm create: self];
  [modelSwarm setObserverSwarm: self];

  CREATE_ARCHIVED_PROBE_DISPLAY (self);
  CREATE_ARCHIVED_PROBE_DISPLAY (modelSwarm);
   
  [controlPanel setStateStopped];

  [modelSwarm buildObjects];


  colorMap = [Colormap create: self];
  [colorMap setColor: 0  ToName: "black"];
  [colorMap setColor: 1  ToName: "white"];
  [colorMap setColor: 2  ToName: "grey"];
  [colorMap setColor: 3  ToName: "yellow"];
  [colorMap setColor: 4  ToName: "pink"];
  [colorMap setColor: 5  ToName: "orange"];
  [colorMap setColor: 6  ToName: "brown"];
  [colorMap setColor: 7  ToName: "red"];
  [colorMap setColor: 8  ToName: "violet"];
  [colorMap setColor: 9  ToName: "green"];
  [colorMap setColor: 10 ToName: "blue"];

  worldRaster = [ZoomRaster createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (worldRaster);
  worldRaster = [worldRaster createEnd];
  [worldRaster setColormap: colorMap];
  [worldRaster setZoomFactor: zoomFactor];
  [worldRaster setWidth: [[modelSwarm getWorld] getSizeX]
                 Height: [[modelSwarm getWorld] getSizeY]];
  [worldRaster setWindowTitle: "Prato Industrial District  1946"];
  [worldRaster pack];

  traderRawMaterialsDisplay = [Object2dDisplay createBegin: self];
  [traderRawMaterialsDisplay setDisplayWidget: worldRaster];
  [traderRawMaterialsDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [traderRawMaterialsDisplay setObjectCollection: [modelSwarm getTraderRawMaterialsList]];
  [traderRawMaterialsDisplay setDisplayMessage: M(drawSelfOn:)];
  traderRawMaterialsDisplay = [traderRawMaterialsDisplay createEnd];

  ragsCollectorDisplay = [Object2dDisplay createBegin: self];
  [ragsCollectorDisplay setDisplayWidget: worldRaster];
  [ragsCollectorDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [ragsCollectorDisplay setObjectCollection: [modelSwarm getRagsCollectorList]];
  [ragsCollectorDisplay setDisplayMessage: M(drawSelfOn:)];
  ragsCollectorDisplay = [ragsCollectorDisplay createEnd];

  carderSpinningDisplay = [Object2dDisplay createBegin: self];
  [carderSpinningDisplay setDisplayWidget: worldRaster];
  [carderSpinningDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [carderSpinningDisplay setObjectCollection: [modelSwarm getCarderSpinningList]];
  [carderSpinningDisplay setDisplayMessage: M(drawSelfOn:)];
  carderSpinningDisplay = [carderSpinningDisplay createEnd];

  comberSpinningDisplay = [Object2dDisplay createBegin: self];
  [comberSpinningDisplay setDisplayWidget: worldRaster];
  [comberSpinningDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [comberSpinningDisplay setObjectCollection: [modelSwarm getComberSpinningList]];
  [comberSpinningDisplay setDisplayMessage: M(drawSelfOn:)];
  comberSpinningDisplay = [comberSpinningDisplay createEnd];

  warperDisplay = [Object2dDisplay createBegin: self];
  [warperDisplay setDisplayWidget: worldRaster];
  [warperDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [warperDisplay setObjectCollection: [modelSwarm getWarperList]];
  [warperDisplay setDisplayMessage: M(drawSelfOn:)];
  warperDisplay = [warperDisplay createEnd];

  weaverDisplay = [Object2dDisplay createBegin: self];
  [weaverDisplay setDisplayWidget: worldRaster];
  [weaverDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [weaverDisplay setObjectCollection: [modelSwarm getWeaverList]];
  [weaverDisplay setDisplayMessage: M(drawSelfOn:)];
  weaverDisplay = [weaverDisplay createEnd];

  dyeingPlantDisplay = [Object2dDisplay createBegin: self];
  [dyeingPlantDisplay setDisplayWidget: worldRaster];
  [dyeingPlantDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [dyeingPlantDisplay setObjectCollection: [modelSwarm getDyeingPlantList]];
  [dyeingPlantDisplay setDisplayMessage: M(drawSelfOn:)];
  dyeingPlantDisplay = [dyeingPlantDisplay createEnd];

  finisherDisplay = [Object2dDisplay createBegin: self];
  [finisherDisplay setDisplayWidget: worldRaster];
  [finisherDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [finisherDisplay setObjectCollection: [modelSwarm getFinisherList]];
  [finisherDisplay setDisplayMessage: M(drawSelfOn:)];
  finisherDisplay = [finisherDisplay createEnd];

  middlemanDisplay = [Object2dDisplay createBegin: self];
  [middlemanDisplay setDisplayWidget: worldRaster];
  [middlemanDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [middlemanDisplay setObjectCollection: [modelSwarm getMiddlemanList]];
  [middlemanDisplay setDisplayMessage: M(drawSelfOn:)];
  middlemanDisplay = [middlemanDisplay createEnd];

  traderFinishedProductsDisplay = [Object2dDisplay createBegin: self];
  [traderFinishedProductsDisplay setDisplayWidget: worldRaster];
  [traderFinishedProductsDisplay setDiscrete2dToDisplay: [modelSwarm getWorld]];
  [traderFinishedProductsDisplay setObjectCollection: [modelSwarm getTraderFinishedProductsList]];
  [traderFinishedProductsDisplay setDisplayMessage: M(drawSelfOn:)];
  traderFinishedProductsDisplay = [traderFinishedProductsDisplay createEnd];


  firstGraph = [Graph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (firstGraph);
  firstGraph = [firstGraph createEnd];
  [firstGraph setTitle: "Variability and Parallelism of Production Chains"];
  [firstGraph setWidth: 500 Height: 200];
  [firstGraph pack];
  [firstGraph setRangesXMin: 1946 Max: 1994 YMin: 0 Max: 1];

  variabilityElement = [firstGraph createElement];
  [variabilityElement setLabel: "Variability"];
  [variabilityElement setColor: "blue"];
  [variabilityElement setWidth: 2];

  parallelismElement = [firstGraph createElement];
  [parallelismElement setLabel: "Parallelism"];
  [parallelismElement setColor: "black"];
  [parallelismElement setWidth: 2];


  secondGraph = [Graph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (secondGraph);
  secondGraph = [secondGraph createEnd];
  [secondGraph setTitle: "Mobility of Finishers and Weavers in Production Chains"];
  [secondGraph setWidth: 500 Height: 200];
  [secondGraph pack];
  [secondGraph setRangesXMin: 1946 Max: 1994 YMin: 0 Max: 1];

  finisherMobilityElement = [secondGraph createElement];
  [finisherMobilityElement setLabel: "Finishers"];
  [finisherMobilityElement setColor: "violet"];
  [finisherMobilityElement setWidth: 2];

  weaverMobilityElement = [secondGraph createElement];
  [weaverMobilityElement setLabel: "Weavers"];
  [weaverMobilityElement setColor: "brown"];
  [weaverMobilityElement setWidth: 2];

  outputMatrix = [OutFile create: self setName: "SimulationResults.txt"];
  outputVariabilityVector = [OutFile create: self setName: "Variability.txt"];
  outputParallelismVector = [OutFile create: self setName: "Parallelism.txt"];
  outputFinisherMobilityVector = [OutFile create: self setName: "FinishersMobility.txt"];
  outputWeaverMobilityVector = [OutFile create: self setName: "WeaversMobility.txt"];
  
  return self;
}




// This method paints black the locations firms have moved away from. Otherwise, firms would leave a trace while moving.

- paintBlack
{
  int x, y;
  for (y=0; y<rasterYSize; y++)
    for (x=0; x<rasterXSize; x++)
      [worldRaster drawPointX: x Y: y Color: 0];
  return self;
}




// This method updates the raster title to the current year. It is activated by -setYearCounter.

- yearInTheTitle: (int)year
{
  char title[50];
  sprintf(title, "Prato Industrial District  %i",year);
  [worldRaster setWindowTitle: title];
  return self;
}




// The following methods communicate to ObserverSwarm at what step and year the simulation is, as well as how many steps there must be during one year. The first two methods are activated by -firmsCreateAndDestroy of ModelSwarm, the third one is activated by -setObserverSwarm of ModelSwarm. Variables stepCounter and yearCounter are used by -communicateChain as well as by -drawCurves. Furthermore, method -setYearCounter updates the raster title.

- setStepCounter: (int)step
{
  stepCounter = step;
  return self;
}

- setYearCounter: (int)year
{
  yearCounter = year;
  [self yearInTheTitle: yearCounter];
  return self;
}

- setStepsInAYear: (int)length
{
  stepsInThisYear = stepsInTheNextYear;
  stepsInTheNextYear = length;

  return self;
}




// This method communicates to ObserverSwarm which production chains are constructed by Middleman, from which side of which Middleman they begin, and which Weaver and which Finisher they entail. It is activated by -step of Middleman.m. Production chains are denoted by the sequence of firms they are made of, where firms are denoted by their conventional numbers. For instance, 9865371 denotes the production chain TraderFinishedProducts-Finisher-Weaver-Warper-CarderSpinning-DyeingPlant-TraderRawMaterials. The 11 possible chains are: 9865371, 98653721, 9865471, 986571, 9865731, 98657321, 9865741, 987651, 9876531, 98765321, 9876541. The numbers denoting the identity and side of Midleman, as well as the identities of Weaver and Finisher, are generated by ModelSwarm. ModelSwarm communicates these numbers to Middleman, Weaver and Finisher, respectively. Then Middleman asks Weaver and Finisher for their identification numbers, and finally passes them on to ObserverSwarm. In order to reset the variables generated herein, -communicateChain is activated even when no chain is constructed.  This method is called several times during each time step, i.e. any time a chain is constructed or the construction of a chain is attempted. Every time step it generates the variables degreeOfParallelism and degreOfVariability, that later on are aggregated by -drawCurves in order to yield yearly values that are depicted on the first graph. Furthermore, every 1000 chains have been constructed, or every year if less than a 1000 are constructed during a year, it calculates "summedFinisherPersistence" and "summedWeaverPersistence". Since this method is called every time a chain is constructed, two variables "oldStepCounter" and "oldYearCounter" have been used to detect when a step or a year is over, respectively. After performing the above calculations, oldStepCounter is set to zero, oldYearCounter is set to zero, and -drawCurves is called.

- communicateChain: (int)chain HangingFrom: (int)hook Including: (int)whichFinisher And: (int)whichWeaver
{
  int i, j;
  int constancy;

  if (oldStepCounter != stepCounter)
    {
      // Here we calculate the degree of parallelism. Every time a new step begins, the ensuing code checks whether a chain appeared at least two times during the previous step. If so, a variable chainXParallelism is set equal to the number of chains X. Subsequently, variables chainXParallelism are averaged over all chains to yield a "degreeOfParallelism". Finally, this "degreeOfParallelism" is added to a "summedDegreeOfParallelism" that is used by -drawCurves to calculate the "averagedDegreeOfParallelism" during one year.

      if (chain9865371Counter >= 2)
	chain9865371Parallelism = chain9865371Counter;
      if (chain98653721Counter >= 2)
	chain98653721Parallelism = chain98653721Counter;
      if (chain9865471Counter >= 2)
	chain9865471Parallelism = chain9865471Counter;
      if (chain986571Counter >= 2)
	chain986571Parallelism = chain986571Counter;
      if (chain9865731Counter >= 2)
	chain9865731Parallelism = chain9865731Counter;
      if (chain98657321Counter >= 2)
	chain98657321Parallelism = chain98657321Counter;
      if (chain9865741Counter >= 2)
	chain9865741Parallelism = chain9865741Counter;
      if (chain987651Counter >= 2)
	chain987651Parallelism = chain987651Counter;
      if (chain9876531Counter >= 2)
	chain9876531Parallelism = chain9876531Counter;
      if (chain98765321Counter >= 2)
	chain98765321Parallelism = chain98765321Counter;
      if (chain9876541Counter >= 2)
	chain9876541Parallelism = chain9876541Counter;

      if (anyChainDuringOneStepCounter == 0)
	degreeOfParallelism = 0.0;
      else
	degreeOfParallelism = ((chain9865371Parallelism + chain98653721Parallelism + chain9865471Parallelism + chain986571Parallelism + chain9865731Parallelism + chain98657321Parallelism + chain9865741Parallelism + chain987651Parallelism + chain9876531Parallelism + chain98765321Parallelism + chain9876541Parallelism) / (double)anyChainDuringOneStepCounter);
      summedDegreeOfParallelism = (summedDegreeOfParallelism + degreeOfParallelism);

      // Here we calculate the degree of variability. First of all, we calculate the constancy of production chains. A variable "constancy" is incremented any time that a production chain is found hanging from the same hook as during the previous step. Subsequently, a "degreeOfVariability" is defined as one minus the ratio of "constancy" to the number of chains that have been constructed during the step. A "summedDegreeOfVariability" sums the "degreeOfVariability" over a year.

      constancy = 0;
      for (i=0; i<anyChainDuringOneStepCounter; i++)
	for (j=0; j<anyChainDuringOneStepCounter; j++)
	  if ((chainDuringOneStep[i] == chainDuringOneStepOld[j]) && (hookDuringOneStep[i] == hookDuringOneStepOld[j]))
	    constancy++;

      if (anyChainDuringOneStepCounter == 0)
	degreeOfVariability = 0.0;
      else
	degreeOfVariability = (1 - ((double)constancy / (double)anyChainDuringOneStepCounter));
      summedDegreeOfVariability = (summedDegreeOfVariability + degreeOfVariability);

      // Here we update the vectors and counters that are used to compute the degree of parallelism and the degree of variability.
      
      for (j=0; j<1000; j++)
	{
	  chainDuringOneStepOld[j] = chainDuringOneStep[j];
	  hookDuringOneStepOld[j] = hookDuringOneStep[j];
	  chainDuringOneStep[j] = 0;
	  hookDuringOneStep[j] = 0;
	}
	  
      chain9865371Counter = 0;
      chain98653721Counter = 0;
      chain9865471Counter = 0;
      chain986571Counter = 0;
      chain9865731Counter = 0;
      chain98657321Counter = 0;
      chain9865741Counter = 0;
      chain987651Counter = 0;
      chain9876531Counter = 0;
      chain98765321Counter = 0;
      chain9876541Counter = 0;
      anyChainDuringOneStepCounter = 0;

      chain9865371Parallelism = 0;
      chain98653721Parallelism = 0;
      chain9865471Parallelism = 0;
      chain986571Parallelism = 0;
      chain9865731Parallelism = 0;
      chain98657321Parallelism = 0;
      chain9865741Parallelism = 0;
      chain987651Parallelism = 0;
      chain9876531Parallelism = 0;
      chain98765321Parallelism = 0;
      chain9876541Parallelism = 0;
    }


  // Here we compute Finisher mobility and Weaver mobility. Firstly, we calculate "persistence". This variable is incremented if over the last 1000 chains a Finisher or Weaver is hanging from the same hook, even if the chains may be qualitatively different. If less than 1000 chains have been constructed during one year, this variables are calculated on less than 1000 chains. Every 1000 chains these variables are summed to yield "summedFinisherPersistence" and "summedWeaverPersistence", respectively. Every year, their values are used by -drawCurves.

  if ((anyChainInTheBufferCounter == 1000) || (oldYearCounter != yearCounter))
    {
      i=0;
      while (i < anyChainInTheBufferCounter)
	{
	  j=0;
	  while (j < anyChainInTheBufferCounter)
	    {
	      if ((whichFinisherDuringOneYear[j] != 0) && ((whichFinisherDuringOneYear[i] == whichFinisherDuringOneYear[j]) && (hookDuringOneYear[i] == hookDuringOneYear[j])) && (i != j))
		{
		  finisherPersistence++;
		  whichFinisherDuringOneYear[j] = 0;
		}
	      if ((whichWeaverDuringOneYear[j] != 0) && ((whichWeaverDuringOneYear[i] == whichWeaverDuringOneYear[j]) && (hookDuringOneYear[i] == hookDuringOneYear[j])) && (i != j))
		{
		  weaverPersistence++;
		  whichWeaverDuringOneYear[j] = 0;
		}
	      j++;
	    }
	  whichFinisherDuringOneYear[i] = 0;
	  whichWeaverDuringOneYear[i] = 0;
	  hookDuringOneYear[i] = 0;
        i++;
	}

      summedFinisherPersistence = (summedFinisherPersistence + finisherPersistence);
      summedWeaverPersistence = (summedWeaverPersistence + weaverPersistence);
      finisherPersistence = 0;
      weaverPersistence = 0;
      anyChainInTheBufferCounter = 0;
    }


  // Here we update "oldStepCounter" and "oldYearCounter". Furthermore, we call -drawCurves.

  if (oldStepCounter != stepCounter)
    oldStepCounter = stepCounter;
  
  if (oldYearCounter != yearCounter)
    {
      [self drawCurves];
      oldYearCounter = yearCounter;
    }


  // Here we update all the counters that are used by -communicateChain and -drawCurves. "anyChainInTheBufferCounter" and "anyChainDuringOneYearCounter" concide if less than 1000 chains are constructed during one year.

  if (chain == 9865371)
    chain9865371Counter++;
  else if (chain == 98653721)
    chain98653721Counter++;
  else if (chain == 9865471)
    chain9865471Counter++;
  else if (chain == 986571)
    chain986571Counter++;
  else if (chain == 9865731)
    chain9865731Counter++;
  else if (chain == 98657321)
    chain98657321Counter++;
  else if (chain == 9865741)
    chain9865741Counter++;
  else if (chain == 987651)
    chain987651Counter++;
  else if (chain == 9876531)
    chain9876531Counter++;
  else if (chain == 98765321)
    chain98765321Counter++;
  else if (chain == 9876541)
    chain9876541Counter++;
  if (chain != 0)
    {
      chainDuringOneStep[anyChainDuringOneStepCounter] = chain;
      hookDuringOneStep[anyChainDuringOneStepCounter] = hook;
      anyChainDuringOneStepCounter++;

      whichFinisherDuringOneYear[anyChainInTheBufferCounter] = whichFinisher;
      whichWeaverDuringOneYear[anyChainInTheBufferCounter] = whichWeaver;
      hookDuringOneYear[anyChainInTheBufferCounter] = hook;
      anyChainInTheBufferCounter++;

      anyChainDuringOneYearCounter++;
    }

  return self;
}




// This method calculates the yearly values of the magnitudes to be drawn on the graph, and calls the graphs elements.  In the case of Parallelism and Variability, it simply aggregates the values that are generated by -communicateChain at each step and draws them on firstGraph. In the case of FinisherMobility and WeaverMobility, it must construct these indices out of "summedFinisherPersistence" and "summedWeaverPersistence". It depicts the ensuing values on firstGraph and secondGraph, respectively, and writes them on a SimulationResults.txt matrix. Once the last point has been drawn, it stops the simulation.

- drawCurves
{
  char outputRow[100];
  char outputVariabilityRow[10];
  char outputParallelismRow[10];
  char outputFinisherMobilityRow[10];
  char outputWeaverMobilityRow[10];

  averagedDegreeOfVariability = (summedDegreeOfVariability / stepsInThisYear);
  averagedDegreeOfParallelism = (summedDegreeOfParallelism / stepsInThisYear);

  [variabilityElement addX: (double)yearCounter Y: (double)averagedDegreeOfVariability];
  [parallelismElement addX: (double)yearCounter Y: (double)averagedDegreeOfParallelism];

  summedDegreeOfVariability = 0.0;
  summedDegreeOfParallelism = 0.0;


  if ((anyChainDuringOneYearCounter == 0) || (summedFinisherPersistence == 0))
    degreeOfFinisherMobility = 0;
  else
    degreeOfFinisherMobility = (1 - (double)summedFinisherPersistence / (double)anyChainDuringOneYearCounter);
  if ((anyChainDuringOneYearCounter == 0) || (summedWeaverPersistence == 0))
    degreeOfWeaverMobility = 0;
  else
    degreeOfWeaverMobility = (1 - (double)summedWeaverPersistence / (double)anyChainDuringOneYearCounter);

  [finisherMobilityElement addX: (double)yearCounter Y: (double)degreeOfFinisherMobility];
  [weaverMobilityElement addX: (double)yearCounter Y: (double)degreeOfWeaverMobility];

  summedFinisherPersistence = 0;
  summedWeaverPersistence = 0;
  anyChainDuringOneYearCounter = 0;
  

  sprintf(outputRow,"%i %f %f %f %f\n", yearCounter, averagedDegreeOfVariability, averagedDegreeOfParallelism, degreeOfFinisherMobility, degreeOfWeaverMobility);
  [outputMatrix putString: outputRow];

  sprintf(outputVariabilityRow,"%f\n",averagedDegreeOfVariability);
  [outputVariabilityVector putString: outputVariabilityRow];

  sprintf(outputParallelismRow,"%f\n",averagedDegreeOfParallelism);
  [outputParallelismVector putString: outputParallelismRow];

  sprintf(outputFinisherMobilityRow,"%f\n",degreeOfFinisherMobility);
  [outputFinisherMobilityVector putString: outputFinisherMobilityRow];

  sprintf(outputWeaverMobilityRow,"%f\n",degreeOfWeaverMobility);
  [outputWeaverMobilityVector putString: outputWeaverMobilityRow];


  if (theEnd == 1)
    {
      [controlPanel setStateStopped];
      [outputMatrix drop];
      [outputVariabilityVector drop];
      [outputParallelismVector drop];
      [outputFinisherMobilityVector drop];
      [outputWeaverMobilityVector drop];
     }

  return self;
}




// This method is activated by main.

- buildActions
{
  [super buildActions];

  [modelSwarm buildActions];

  displayActions = [ActionGroup create: self];
  [displayActions createActionTo: self message: M(paintBlack)];
  [displayActions createActionTo: traderRawMaterialsDisplay   message: M(display)];
  [displayActions createActionTo: ragsCollectorDisplay   message: M(display)];
  [displayActions createActionTo: carderSpinningDisplay   message: M(display)];
  [displayActions createActionTo: comberSpinningDisplay   message: M(display)];
  [displayActions createActionTo: warperDisplay   message: M(display)];
  [displayActions createActionTo: weaverDisplay   message: M(display)];
  [displayActions createActionTo: dyeingPlantDisplay   message: M(display)];
  [displayActions createActionTo: finisherDisplay   message: M(display)];
  [displayActions createActionTo: middlemanDisplay   message: M(display)];
  [displayActions createActionTo: traderFinishedProductsDisplay   message: M(display)];
  [displayActions createActionTo: worldRaster   message: M(drawSelf)];
  [displayActions createActionTo: actionCache   message: M(doTkEvents)];
  [displayActions createActionTo: probeDisplayManager message: M(update)];

  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];
 
  return self;
}




- activateIn: swarmContext
{
  [super activateIn: swarmContext];
  [modelSwarm activateIn: self];
  [displaySchedule activateIn: self];
  return [self getSwarmActivity];
}




// This method ends the simulation when input data are finished. It is activated by -firmsCreateAndDestroy of ModelSwarm.m. This method does not directly operate the command [controlPanel setStateStopped] because the last dots would not be traced on the graphs. Command [controlPanel setStateStopped] is operated by -drawCurves just after the graph elements have been called.

- end
{
  theEnd = 1;
  return self;
}




// This method stops the simulation. It is used when warnings are produced by ModelSwarm, because the user chose illegal parameters.

- stop
{
  [controlPanel setStateStopped];
  return self;
}




@end


















