// Warper.m

//Copyright (C) 2001 by Guido Fioretti, via Angeloni 11, I - 05100 Terni. This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation.

#import "Warper.h"
#import "ModelSwarm.h"
#import <random.h>






@implementation Warper




// This method is used by ModelSwarm to initialize a Warper's position, as well as by Middleman to move it into a production chain. Global variables xPosEast, xPosWest, yPosNorth, yPosSouth, xEastEast, yNorthNorth, xWestWest and ySouthSouth are initialized here, as well as global addresses eastOfWarper, northOfWarper, westOfWarper, southOfWarper, eastEastOfWarper, northNorthOfWarper, westWestOfWarper, southSouthOfWarper, northEastOfWarper, northWestOfWarper, southWestOfWarper and southEastOfWarper.

- setX: (int)x Y: (int)y
{
  [world putObject: nil atX: xPos Y: yPos];
  xPos = x;
  yPos = y;
  [world putObject: self atX: xPos Y: yPos];

  xPosEast = (xPos + 1 + worldXSize) % worldXSize;
  yPosNorth = (yPos + 1 + worldYSize) % worldYSize;
  xPosWest = (xPos - 1 + worldXSize) % worldXSize;
  yPosSouth = (yPos - 1 + worldYSize) % worldYSize;
  
  xPosEastEast = (xPosEast + 1 + worldXSize) % worldXSize;
  yPosNorthNorth = (yPosNorth + 1 + worldYSize) % worldYSize;
  xPosWestWest = (xPosWest - 1 + worldXSize) % worldXSize;
  yPosSouthSouth = (yPosSouth - 1 + worldYSize) % worldYSize;
  
  eastOfWarper = [world getObjectAtX: xPosEast Y: yPos];
  northOfWarper = [world getObjectAtX: xPos Y: yPosNorth];
  westOfWarper = [world getObjectAtX: xPosWest Y: yPos];
  southOfWarper = [world getObjectAtX: xPos Y: yPosSouth];

  eastEastOfWarper = [world getObjectAtX: xPosEastEast Y: yPos];
  northNorthOfWarper = [world getObjectAtX: xPos Y: yPosNorthNorth];
  westWestOfWarper = [world getObjectAtX: xPosWestWest Y: yPos];
  southSouthOfWarper = [world getObjectAtX: xPos Y: yPosSouthSouth];

  northEastOfWarper = [world getObjectAtX: xPosEast Y: yPosNorth];
  northWestOfWarper = [world getObjectAtX: xPosWest Y: yPosNorth];
  southWestOfWarper = [world getObjectAtX: xPosWest Y: yPosSouth];
  southEastOfWarper = [world getObjectAtX: xPosEast Y: yPosSouth];
  
  return self;
}




// These methods make xPos, yPos, xPosEast, xPosWest, yPosNorth and yPosSouth available to other firms.

- (int)getXPos
{
  return xPos;
}

- (int)getYPos
{
  return yPos;
}

- (int)getXPosEast
{
  return xPosEast;
}

- (int)getXPosWest
{
  return xPosWest;
}

- (int)getYPosNorth
{
  return yPosNorth;
}

- (int)getYPosSouth
{
  return yPosSouth;
}




// This method enables ModelSwarm to give an identity to each instance of Warper that it creates.

- setIdentity: (int)ID
{
  identity = ID;
  return self;
}




// This method allows Middleman to ask Warper about its identity.

- (int)getIdentity
{
  return identity;
}




// This method enables ModelSwarm to make the grid visible to Warper. As soon as the grid is visible, Warper imports its size.

- setWorld: W
{
  world = W;

  worldXSize = [world getSizeX];
  worldYSize = [world getSizeY];
  
  return self;
}




// This method enables Warper to see ModelSwarm. In this way, it can ask ModelSwarm for the positions where production chains are, and not jump into them.  In order to avoid cross-references, ModelSwarm.h is imported in Warper.m. This method is activated by -firmsCreateAndDestroy of ModelSwarm.

- setModelSwarm: MS
{
  modelSwarm = MS;
  return self;
}




// This method ends the creation of Warper. It can be used to initialize global variables.

- createEnd
{
  [super createEnd];
  
  workInProgress = 0;
  
  return self;
}




// When asked who they are, Warper answer "5".

- (int)whoAreYou
{
  return 5;
}




// These methods say whether the positions surrounding Warper are free.  Method -doYouHaveSpaceAround returns 1 if at least one out of the four positions surrounding Warper is free and it is surrounded by free positions, 0 otherwise. Method -doYouHaveFreeSpaceEast returns 1 if the position east of Warper as well as the three positions that surround it are free, 0 otherwise. Methods -doYouHaveFreeSpaceNorth, -doYouHaveFreeSpaceWest and -doYouHaveFreeSpaceSouth behave similarly.

- (int)doYouHaveFreeSpaceAround
{
  int freeSpaceAround;

  if (((eastOfWarper == nil) && (southEastOfWarper == nil) && (eastEastOfWarper == nil) && (northEastOfWarper == nil)) || ((northOfWarper == nil) && (northEastOfWarper == nil) && (northNorthOfWarper == nil) && (northWestOfWarper == nil)) || ((westOfWarper == nil) && (southWestOfWarper == nil) && (westWestOfWarper == nil) && (northWestOfWarper == nil)) || ((southOfWarper == nil) && (southWestOfWarper == nil) && (southSouthOfWarper == nil) && (southEastOfWarper == nil)))
    freeSpaceAround = 1;
  else
    freeSpaceAround = 0;

  return freeSpaceAround;
}

- (int)doYouHaveFreeSpaceEast
{
  int freeSpaceEast;

  if ((eastOfWarper == nil) && (southEastOfWarper == nil) && (eastEastOfWarper == nil) && (northEastOfWarper == nil))
    freeSpaceEast = 1;
  else
    freeSpaceEast = 0;
  
  return freeSpaceEast;
}

- (int)doYouHaveFreeSpaceNorth
{
  int freeSpaceNorth;

  if ((northOfWarper == nil) && (northEastOfWarper == nil) && (northNorthOfWarper == nil) && (northWestOfWarper == nil))
    freeSpaceNorth = 1;
  else
    freeSpaceNorth = 0;
  
  return freeSpaceNorth;
}

- (int)doYouHaveFreeSpaceWest
{
  int freeSpaceWest;

  if ((westOfWarper == nil) && (northWestOfWarper == nil) && (westWestOfWarper == nil) && (southWestOfWarper == nil))
    freeSpaceWest = 1;
  else
    freeSpaceWest = 0;
  
  return freeSpaceWest;
}

- (int)doYouHaveFreeSpaceSouth
{
  int freeSpaceSouth;

  if ((southOfWarper == nil) && (southWestOfWarper == nil) && (southSouthOfWarper == nil) && (southEastOfWarper == nil))
    freeSpaceSouth = 1;
  else
    freeSpaceSouth = 0;
  
  return freeSpaceSouth;
}




// This method allows Warper to receive the values of workInProgress that other firms may want to communicate. In -createEnd the variable "workInProgress" is initialized to 0.

- setWorkInProgress: (int)wp
{
  workInProgress = wp;
  return self;
}




// This method allows Warper to tell other firms whether it is working. This information is carried by the 0-th component of the workingState vector. The other components are only used by the -areYouWorking method of Middleman.

- (int *)areYouWorking
{
  workingState[0] = workInProgress;
  workingState[1] = 0;
  workingState[2] = 0;
  workingState[3] = 0;
  workingState[4] = 0;

  return workingState;
}




// Firms are depicted on a torus.  In order to understand the "step" method, please remember that the raster draws positions in [0, (worldXSize - 1)] X [0, (worldYSize - 1)]. Unless called by a Middleman, Warper move around according to a random walk. In order to exit the observation range of other firms, jumps are not shorter than 2 pixels. Before moving into a new position, Warper checks that it is not booked for constructing a production chain. Furthermore, in order not to enter the observation range of other firms, Warper only moves into a new position if its 12 nearest neighbors are free (east, north, west, south, northEast, northWest, southWest, southEast, eastEast, northNorth, westWest, southSouth). The length of a jump is gradually increased until a free position is found and the jump is actually carried out. Positions are examined along the borders of a square centered around the current position, gradually expanding from 2x2 onwards. The square can accomodate to unequal X and Y axes. Along the borders of the square, positions are examined according to a uniform distribution. The number of trials along a border of the square is equal to the length of the border itself. The square can expand up to the raster size; if no free position when the square becomes as large as the raster, a new search begins from a 2x2 square (this makes sense, because random examination of the borders of the square does not ensure that all positions are examined). This procedure avoids using memory-consuming normal distributions.

- step
{
  int widthX = 2;
  int widthY = 2;
  int stepX = 0;
  int stepY = 0;
  int xNewPos = xPos;
  int yNewPos = yPos;
  int xNewPosEast = xPosEast;
  int yNewPosNorth = yPosNorth;
  int xNewPosWest = xPosWest;
  int yNewPosSouth = yPosSouth;
  int xNewPosEastEast = xNewPosEast;
  int yNewPosNorthNorth = yNewPosNorth;
  int xNewPosWestWest = xNewPosWest;
  int yNewPosSouthSouth = yNewPosSouth;
  id inNewPosition;
  id eastOfNewPosition;
  id northOfNewPosition;
  id westOfNewPosition;
  id southOfNewPosition;
  id northEastOfNewPosition;
  id northWestOfNewPosition;
  id southWestOfNewPosition;
  id southEastOfNewPosition;
  id eastEastOfNewPosition;
  id northNorthOfNewPosition;
  id westWestOfNewPosition;
  id southSouthOfNewPosition;
  int tryXorY;
  int flip;
  int flop;
  int trialX = 0;
  int trialY = 0;
  int jumped = 0;
    
  if (workInProgress == 0)
    {
      // If it is not working, a Warper makes a jump of length 2, or higher.  When Warper is leaving a production chain, any shorter jump would keep it inside the range of the -doYouHaveFreeSpace method of the previous link of the chain.  Jumping is particularly important if Warper was working during the previous time interval, because it gives the system some probability to modify its production chains. However, it is also important if Warper was just jumping around, since in this way it can be captured by a different Middleman.

      tryXorY = [uniformIntRand getIntegerWithMin: 0 withMax: 1];
      flip = [uniformIntRand getIntegerWithMin: 0 withMax: 1];
      flop = [uniformIntRand getIntegerWithMin: 0 withMax: 1];

      while (jumped == 0)
	{
	  if (tryXorY == 0)
	    {
	      trialX++;
	      if (trialX == widthX)
		{
		  widthX++;
		  if (widthX == worldXSize)
		    widthX = 2;
		  trialX = 0;
		}
	      while ((stepX == -1) || (stepX == 0) || (stepX == 1))
		stepX = [uniformIntRand getIntegerWithMin: -widthX withMax: widthX];
	      xNewPos = xPos + stepX;
	      if (flip == 0)
		yNewPos = yPos - widthY;
	      else if (flip == 1)
		yNewPos = yPos + widthY;
	      if (flip == 0)
		flip = 1;
	      else if (flip == 1)
		flip = 0;
	    }
	  else if (tryXorY == 1)
	    {	  
	      trialY++;
	      if (trialY == widthY)
		{
		  widthY++;
		  if (widthY == worldYSize)
		    widthY = 2;
		  trialY = 0;
		}
	      if (flop == 0)
		xNewPos = xPos - widthX;
	      else if (flop == 1)
		xNewPos = xPos + widthX;
	      while ((stepY == -1) || (stepY == 0) || (stepY == 1))
		stepY = [uniformIntRand getIntegerWithMin: -widthY withMax: widthY];
	      yNewPos = yPos + stepY;
	      if (flop == 0)
		flop = 1;
	      else if (flop == 1)
		flop = 0;
	    }
	  
	  if (xNewPos < 0)
	    xNewPos = (xNewPos + worldXSize); 
	  else if (xNewPos >= worldXSize)
	    xNewPos = (xNewPos - worldXSize);
	  if (yNewPos < 0)
	    yNewPos = (yNewPos + worldYSize);
	  else if (yNewPos >= worldYSize)
	    yNewPos = (yNewPos - worldYSize);
	  
	  xNewPosEast = (xNewPos + 1 + worldXSize) % worldXSize;
	  yNewPosNorth = (yNewPos + 1 + worldYSize) % worldYSize;
	  xNewPosWest = (xNewPos - 1 + worldXSize) % worldXSize;
	  yNewPosSouth = (yNewPos - 1 + worldYSize) % worldYSize;      
	  xNewPosEastEast = (xNewPosEast + 1 + worldXSize) % worldXSize;
	  yNewPosNorthNorth = (yNewPosNorth + 1 + worldYSize) % worldYSize;
	  xNewPosWestWest = (xNewPosWest - 1 + worldXSize) % worldXSize;
	  yNewPosSouthSouth = (yNewPosSouth - 1 + worldYSize) % worldYSize;
	  
	  inNewPosition = [world getObjectAtX: xNewPos Y: yNewPos];
	  eastOfNewPosition = [world getObjectAtX: xNewPosEast Y: yNewPos];
	  northOfNewPosition = [world getObjectAtX: xNewPos Y: yNewPosNorth];
	  westOfNewPosition = [world getObjectAtX: xNewPosWest Y: yNewPos];
	  southOfNewPosition = [world getObjectAtX: xNewPos Y: yNewPosSouth];
	  northEastOfNewPosition = [world getObjectAtX: xNewPosEast Y: yNewPosNorth];
	  northWestOfNewPosition = [world getObjectAtX: xNewPosWest Y: yNewPosNorth];
	  southWestOfNewPosition = [world getObjectAtX: xNewPosWest Y: yNewPosSouth];
	  southEastOfNewPosition = [world getObjectAtX: xNewPosEast Y: yNewPosSouth];
	  eastEastOfNewPosition = [world getObjectAtX: xNewPosEastEast Y: yNewPos];
	  northNorthOfNewPosition = [world getObjectAtX: xNewPos Y: yNewPosNorthNorth];
	  westWestOfNewPosition = [world getObjectAtX: xNewPosWestWest Y: yNewPos];
	  southSouthOfNewPosition = [world getObjectAtX: xNewPos Y: yNewPosSouthSouth];
	  
	  if (([modelSwarm isBookedX: xNewPos Y: yNewPos] == 0) && (inNewPosition == nil) && (eastOfNewPosition == nil) && (northOfNewPosition == nil) && (westOfNewPosition == nil) && (southOfNewPosition == nil) && (northEastOfNewPosition == nil) && (northWestOfNewPosition == nil) && (southWestOfNewPosition == nil) && (southEastOfNewPosition == nil) && (eastEastOfNewPosition == nil) && (northNorthOfNewPosition == nil) && (westWestOfNewPosition == nil) && (southSouthOfNewPosition == nil))
	    {
	      [world putObject: nil atX: xPos Y: yPos];
	      xPos = xNewPos;
	      yPos = yNewPos;
	      [world putObject: self atX: xPos Y: yPos];
	      jumped = 1;
	    }
	  else
	    {
	      stepX = 0;
	      stepY = 0;
	      if (tryXorY == 0)
		tryXorY = 1;
	      else if (tryXorY == 1)
		tryXorY = 0;
	    }
	  
	}

      // Since Warper moved, global variables xPosEast, xPosWest, yPosNorth and yPosSouth must be updated.      

      xPosEast = (xPos + 1 + worldXSize) % worldXSize;
      yPosNorth = (yPos + 1 + worldYSize) % worldYSize;
      xPosWest = (xPos - 1 + worldXSize) % worldXSize;
      yPosSouth = (yPos - 1 + worldYSize) % worldYSize;
      
      xPosEastEast = (xPosEast + 1 + worldXSize) % worldXSize;
      yPosNorthNorth = (yPosNorth + 1 + worldYSize) % worldYSize;
      xPosWestWest = (xPosWest - 1 + worldXSize) % worldXSize;
      yPosSouthSouth = (yPosSouth - 1 + worldYSize) % worldYSize;
    }


  // As long as a Warper is working, it doesn't move from its position.

  else if (workInProgress == 1)
    {
      [world putObject: nil atX: xPos Y: yPos];
      [world putObject: self atX: xPos Y: yPos];
    }


  // Whether Warper moved or not, its neighbors might have changed and must be updated.

  eastOfWarper = [world getObjectAtX: xPosEast Y: yPos];
  northOfWarper = [world getObjectAtX: xPos Y: yPosNorth];
  westOfWarper = [world getObjectAtX: xPosWest Y: yPos];
  southOfWarper = [world getObjectAtX: xPos Y: yPosSouth];

  eastEastOfWarper = [world getObjectAtX: xPosEastEast Y: yPos];
  northNorthOfWarper = [world getObjectAtX: xPos Y: yPosNorthNorth];
  westWestOfWarper = [world getObjectAtX: xPosWestWest Y: yPos];
  southSouthOfWarper = [world getObjectAtX: xPos Y: yPosSouthSouth];

  northEastOfWarper = [world getObjectAtX: xPosEast Y: yPosNorth];
  northWestOfWarper = [world getObjectAtX: xPosWest Y: yPosNorth];
  southWestOfWarper = [world getObjectAtX: xPosWest Y: yPosSouth];
  southEastOfWarper = [world getObjectAtX: xPosEast Y: yPosSouth];


  return self;
}




// This method draws a Warper on the raster.

- drawSelfOn: (id <Raster>)R
{
  [R drawPointX: xPos Y: yPos Color: 5];
  return self;
}




@end  



















